!**************************************************************
! module omphotometry_mod (10th July 2001)
!
! Module containing routines, etc,
! for doing aperture photometry on each source in the list. It has been added
! because the counts that initially computed for each source (at the same
! time that the centroided positions for each source are obtained) is not
! consistent with the CAL calibration of source counts to magnitudes.
! It also sets the source confusion flag and the source
! quality flag. 
!
!**************************************************************
module omPhotometry_mod
    
    use param
    use cal
    use dal
    use OmAllocation_mod      ! For memory allocation
    use omconstants_mod
    use OmPhotometryDefs_mod
    use OmNumerics_mod
    use OmSkyBackground_mod   ! To compute sky background
    use OmSourceparam_mod
    use OmCoinccorr_mod       ! For coincidence-loss corrections   
    use OmSmoothImage_mod
    use omellipse_mod    
interface GetExtendedSourceCounts
    module procedure GetExtendedSourceCounts1, &
                     GetExtendedSourceCounts2
end interface


interface GetExtendedCountRate
    module procedure GetExtendedCountRate1,  &
                     GetExtendedCountRate2
end interface





interface GetBackgroundMode
    module procedure GetBackgroundMode1, &
                     GetBackgroundMode2, &
                     GetBackgroundMode3, &
                     GetBackgroundMode4


end interface
interface GetCounts
    module procedure GetCounts1, GetCounts2
end interface

interface GetBackgroundCounts
    module procedure GetBackgroundCounts1


end interface


interface AperturePhotometry
module procedure AperturePhotometry1, &
                 AperturePhotometry2, &
                 AperturePhotometry3, &
                 AperturePhotometry4


end interface AperturePhotometry
   
interface ComputeSourceRateError
    module procedure ComputeSourceRateError1

end interface ComputeSourceRateError





interface setQualityFlag
    module procedure setQualityFlag1, setQualityFlag2
end interface setQualityFlag



type(SourceParameters), pointer, private :: currentSource => null()









       
real(single), private, parameter :: backgroundSigma = 2.5
real(single), private :: areaRatio = 3.0  ! WAS 3.0
integer, parameter, private :: minData = 100 ! was 299





!**********************************************************
!
!
!***********************************************************





!****************************************************************
!
! The following are only used when there is an exposure image
!
!****************************************************************

real(double), private :: meanExposure, minExposure, maxExposure
integer, private :: numExposurePixels

 
real(single), private :: exposureConstant = 1.0

contains               

!*************************************************************
! Sets the number of pixels to be used in the background 
! determination to be ratio * the number of pixels in the
! aperture  
!
!*************************************************************
subroutine setAreaRatio( ratio )
    real(single), intent(in) :: ratio

    areaRatio = ratio
end subroutine setAreaRatio

!*************************************************************
! 
!*************************************************************
real(single) function getAreaRatio()
   

    getAreaRatio = areaRatio

end Function getAreaRatio

!***************************************************************************
! If want to use an exposure image then need to set it
! using this routine 
!
!
!***************************************************************************/
subroutine setExposureImage( image, constant )
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: constant
    exposureImage => image
    exposureConstant = constant

end subroutine setExposureImage

!****************************************************************************************
! subroutine AperturePhotometry
! Performs aperture photometry on a set of sources!
! using a circular aperture of a given pixel radius
! Parameters-
!     nx and ny - the dimensions of the images
!     image - the image
!     sourceQualityImage - describes the quality of each image pixel
!     photometryRadius - The aperture radius for doing the photometry
!     noSources - number of sources
!     xSource - array of source x-positions
!     ySource - array of source y-positions
!     psfXwidth - array holding the computed x extents of the sources (pixels).
!     psfYwidth - array holding the computed y extents of the sources (pixels).y
!     fSource - array to hold the counts for each source.
!               Contains the Jon Mittaz counts on entry
!               and where possible these will be replaced 
!               by the aperture photometry values.!
!     fBackGround - array to hold the background 
!               counts for each source. 
!               Contains the Jon Mittaz counts on entry
!               and where possible these will be replaced 
!               by the aperture photometry values.
!     sourceExtendedFlag - array containg either 0 (not extended source)
!                or 1 (extended source)
!     sourceConfusionFlag - array containing either 1 - no source within
!                           2, source within aperture but outside minimum aperture
!                           radius, or 3 - source within minimum apertuire radius.
!     sourceQualityFlag - Set to 1 if source region has a bad-quality pixel- 0 otherwise
!     sourceApertureRadius - the aperture radius that was used for     
!                      each source 
!****************************************************************************************

subroutine AperturePhotometry1(   &
    nx1,                     &  ! The x-extent of the images 
    ny1,                     &  ! The y-extent of the images 
    image,                   &  ! The image (including background) 
    backgroundImageIn,       &  ! The background image (if any)
    levelImage,              &  ! The source number image
    sourceQualityImage,      &  ! The pixel-quality image array 
    photometryRadius,        &  ! The aperture radius (pixels) to be used for doing photometry
    noSources,               &  ! The number of sources
    source)                     ! Array of type SourceParamters structures

    ! Input and unmodified variables
    integer(int16), intent(in) :: nx1, ny1                      
    real(single), pointer              :: image(:, :)   
    real(single), pointer    :: backGroundImageIn(:, :)   
    integer(int16), pointer           :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
    real(single), intent(in) :: photometryRadius       
    integer (int16), intent(in)                      :: noSources

    ! The following structure is defined in  "omdetectsource_mod.f90"
    ! It stores all the parameters associated with a source 
   
    type(SourceParameters), pointer :: source(:)       

    backGroundImage => backgroundImageIn 
 

    call AperturePhotometry2(    &
        nx1,                     &  ! The x-extent of the images 
        ny1,                     &  ! The y-extent of the images 
        image,                   &  ! The image (including background) 
        levelImage,              &  ! The source number image
        sourceQualityImage,      &  ! The pixel-quality image array 
        photometryRadius,        &  ! The aperture radius (pixels) to be used for doing photometry
        noSources,               &  ! The number of sources
        source)                     ! Array of type SourceParamters structures



    nullify(backgroundImage)

end subroutine AperturePhotometry1
!****************************************************
!
!
!****************************************************
subroutine AperturePhotometry2(   &
    nx1,                     &  ! The x-extent of the images 
    ny1,                     &  ! The y-extent of the images 
    image,                   &  ! The image (including background) 
    levelImage,              &  ! The source number image
    sourceQualityImage,      &  ! The pixel-quality image array 
    photometryRadius,        &  ! The aperture radius (pixels) to be used for doing photometry
    noSources,               &  ! The number of sources
    source)                     ! Array of type SourceParamters structures

    ! Input and unmodified variables
    integer(int16), intent(in) :: nx1, ny1                      
    real(single), pointer              :: image(:, :)      
    integer(int16), pointer           :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
    real(single), intent(in) :: photometryRadius       
    integer (int16), intent(in)                      :: noSources

    ! The following structure is defined in  "omdetectsource_mod.f90"
    ! It stores all the parameters associated with a source 
   
    type(SourceParameters), pointer :: source(:)       
 
    ! Local variables

    integer(int32)           :: i, k, status, type    
    real(single) :: maxRadius         
    real(single) :: backGroundCountsError
    real(single) :: sigma    

    ! NB Set minSignificance to zero if no counts extrapolation
    ! to be used ! NB until the CAL calibration data is in place
    ! the full aperture will be used on all sources, unless
    ! another source lies within the aperture. If the source 
    ! significance is less than this value then
    ! the photometric counts will be extrapolated using
    ! the point-spread-function
    real(single), parameter :: minSignificance = 0.0  
    integer (int32) :: index

!integer, save :: count = 0       
 
    !write(*,*) "L285 aperture photometry"
    if (noSources .lt. 1) return ! Return if no sources


!    write(errstr,'(a)') "Performing aperture photometry on the sources"
!    call message(AppMsg, Verbose)
 

    ! Initialise arrays, etc
    apertureRadius = photometryRadius
    

    call Initialise(noSources, nx1, ny1)


    !******************************************************************
    !
    !
    !
    !******************************************************************

    CCDImage => image  ! Assign to input image

    qualityImage => sourceQualityImage ! The quality image (false=good pixels, true=bad pixels)
   
    
    sourceNumImage => levelImage ! The source number image

      
    call refineSourceNumImage(source, nosources, sourceNumimage)


    ! Set the minimum aperture radius to half the required aperture
    minimumApertureRadius = photometryRadius * 0.5
    fullApertureArea = PI * photometryRadius * photometryRadius  

    
    !**********************************************************
    ! Obtain an index of the source positions in
    ! increasing x-pixel position. NB xSource is not
    ! sorted. sortIndex(i) will give the index of the 
    ! array element that gives the positions in increasing
    ! x-position. This is used for obtaining the distance
    ! from one source to its nearest neighbour. 
    !***********************************************************
    call IndexSort(source%xPos, sortIndex, int(noSources, int32))  





    ! Compute the mode of the image, ignoring source regions

    backgroundCountsMode = ComputeBackgroundMode(image, &
                     sourceNumImage, nx, ny, backgroundSigma, sigma)


    backgroundCountsModeError = sigma


    ! sourceCoords will contain the position of the sources
    ! in increasing x-position- this array is used to obtain
    ! the maximum-sized aperture for doing photometry.
   
    do i = 1, noSources
        sourceCoords(i)%x = source(sortIndex(i))%xPos - 0.5   
        sourceCoords(i)%y = source(sortIndex(i))%yPos - 0.5  
        sourceCoords(i)%extension = source(sortIndex(i))%extensionFlag
!        sourceCoords(i)%maxCounts =  abs( source(sortIndex(i))%rawCountRate )
        ! objectWidth will store the maximum of the x and y extents of the sources
        ! * 3, or if this is less than the aperture radius, to the aperture radius.                       
        objectWidth(i) = photometryRadius   

    end do
  
    ! Loop through all the sources doing aperture photometry on each one (if possible)

  
    k = 0







!write(*, *) "********************************************************************************"
!write(*, *) "********************************************************************************"
!write(*, *) "********************************************************************************"
!write(*, *) "********************************************************************************"
!write(*, *) "********************************************************************************"
!write(*, *) "********************************************************************************"




    do i = 1,  noSources ! Remember photometry on sources is conducted in increasing x-order        
        index = sortIndex(i)
        sourceNumber = source(index)%number

        currentSource => source(index)         
        currentSource%number = sourceNumber
        !write(*,*) "***********************************"
        !write(*,*) "L388 i=", " extFlag=", currentSource%extensionFlag, " rawCountRate=", &
        !     currentSource%rawCountRate
        
! if (abs(currentSource%xpos - 388.0) > 3.0 .and. abs(currentSource%ypos-273.0) > 3.0) cycle     

        if (currentSource%extensionFlag .eq. 0 ) then  

            ! Point-source photometry using circular aperture
            
            apertureRadius = photometryRadius    
            !write(*,*) "L396 apertureRadius=", apertureRadius, " coincLossRadius=", &
            !     coincLossRadius
            !********************************************
            ! Find the aperture radius that can be used
            ! for the photometry. If type is 1 then
            ! can use the required aperture, 2 - have
            ! to use a smaller one, 3 can't do photometry
            !********************************************


            if (currentSource%apertureRadius > coincLossradius) then
                type = 5

            else if (currentSource%apertureRadius .gt. 0.0) then


                maxRadius = GetMaximumAperture(noSources, i, type)          
                
                maxRadius = currentSource%apertureRadius
                type = 1

               
            else
                !***********************************************************
                ! Basically the default in imaging chain
                !***********************************************************
                maxRadius = GetMaximumAperture(noSources, i, type)


                if (btest(currentSource%qualityFlag, 4)) then 
                    maxRadius = coincLossRadius          
                end if 

            end if 

            !write(*, *) "Line 431 type=", type, " apertureradius=", maxradius                         

            select case (type)    
                case (1, 2, 4)     ! 1 - No other objects within aperture, 
                                   ! 2 - Another object within aperture, but outside 
                                   ! minimum aperture radius- use aperture that 
                                   ! excludes this source
                                                     

                    maxradius = max(maxRadius, coincLossRadius / 2.0)
                    ! Point-source photometry


 
                    if(associated(exposureImage) ) then                                      
                            
                        call PointSourcePhotometry4( currentSource, maxRadius)                                       

                   else
                        call PointSourcePhotometry1(currentSource, maxRadius) 
                    end if


                    if (type .eq. 4) then ! Source lies near an image edge
                        fullApertureArea = PI * maxRadius * maxRadius



!write(*, *) "Aperture photometry- source lies near an edge"                  
!write(*, *) "x=", currentSource%xpos, " y=", currentSource%ypos, &
!" apertureArea", apertureArea, " full=", fullApertureArea
!write(*, *) "maxradius=", maxradius, " coinclossradius=", coinclossradius
!write(*, *) "rawbackgroundcountrate=",   currentSource%rawBackGroundCountRate
                        currentSource%rawCountRate = currentSource%rawCountRate * fullApertureArea / apertureArea 
                        currentSource%rawCountRateErr = currentSource%rawCountRateErr * fullApertureArea / apertureArea 
 
                     
                        currentSource%rawBackGroundCountRate = currentSource%rawBackGroundCountRate &
                            * fullApertureArea / apertureArea 
                        backGroundCountsError = backGroundCountsError * fullApertureArea / apertureArea 
                        currentSource%rawbackgroundCountRateErr =  backGroundCountsError / currentSource%exposure
                
                        currentSource%apertureRadius = maxRadius



                    end if
                 

                    !***********************************************************************
                    !
                    ! If using an exposure image then multiply significance by sqrt(mean exposure)
                    !
                    !*************************************************************************
!                    if (associated(exposureImage) ) then
!                        call scaleErrors( currentSource )        
!                    end if

   
                    currentSource%ApertureRadius = maxRadius * real(binX)
                                
                    ! Correct counts for coincidence losses
                    ! NB The corrected count rates are stored in source
                    ! members correctedCountRate, correctedCountRateErr,
                    ! and correctedBackgroundRateErr   

                   
                    call PointSourceCoincLossCorr( currentSource, 12.0, frameTime )     
                  

                case (3)   

                   
                    maxRadius = coinclossRadius / 2.0                    



                    if(associated(exposureImage) ) then                        
                        call PointSourcePhotometry4(currentSource, maxRadius) 
                    else
                      
                        call PointSourcePhotometry1(currentSource, maxRadius) 
                    end if




                    currentSource%ApertureRadius = maxRadius * real(binX)


                    !***********************************************************************
                    !
                    ! If using an exposure image then multiply significance by sqrt(mean exposure)
                    !
                    !*************************************************************************
!                    if (associated(exposureImage) ) then
!                        call scaleErrors( currentSource )
!                    end if

                    call PointSourceCoincLossCorr(currentSource, 12.0, frameTime)     

                    type = 3
                     
                case (5)  ! Extended photometry on point-source (ie aperture bigger
                          ! than coincidence-loss aperture


                    call PointSourcePhotometry2(currentSource, currentSource%apertureRadius)

     
            end select
         else

            ! Extended source photometry
            ! The counts (source + background) are corrected
            ! for coincidence-losses  
   
            !write(*,*) "L549 extended source photometry" 

            if(associated(exposureImage) ) then         
               !write(*,*) "L552 before get ExtendedSourcePhotometry4 "
               call ExtendedSourcePhotometry4(currentSource) 
            else
                      
               !write(*,*) "L557 i=", " apertureRadius=", currentSource%apertureRadius, " coincLossRadius=", &
               !     coincLossRadius

               if (currentsource%apertureRadius .le. 0.0) then
                  call GetExtendedSourceCounts(currentSource, coincLossRadius)
                  !write(*,*) "L562 "
               else
                  ! For elliptical apertures
                  !write(*,*) "L561  omphotometry: before getExtendedSourceCounts"
                  call GetExtendedSourceCounts(currentSource, coincLossRadius,   &
                       currentSource%apertureRadius * currentSource%axialRatio, &   ! semi a
                       currentSource%apertureRadius, &                              ! semi b
                       currentSource%alpha)
               end if
            end if
                       
            type = 0
         end if



!if (count .lt. 20) then
!count = count + 1

!source(index)%semia = source(index)%apertureRadius / 2.0
!source(index)%semib = source(index)%semia

 
!write(*, *) i, " )Line 403 index=", index, " source x=", source(index)%xpos, " y=", source(index)%ypos, " sig=", &
!" rawrate=", source(index)%rawcountrate, " rawbackgroundrate=", source(index)%rawbackgroundcountrate,      &
!" corrcountrate=", source(index)%corrCountRate, " corrbackgroundrate=", source(index)%corrbackgroundcountrate, &
!" significance=", source(index)%significance, " type=", type, " eflag=", source(index)%extensionFlag, &
!" qflag=", source(index)%qualityflag, " cflag=", source(index)%confusionflag, &
!" apertureradius=", source(index)%apertureradius
!end if

        ! Set the source confusion flag - 0 isolated source
        ! 1 Source within aperture radius and greater than minimum aperture radius
        ! 2 Source within minimum aperture radius         
        nullify(currentSource)        





!100 format(i8, 8e12.4, f12.2, i8, i8, f8.1, i6)
    end do


!write(*, *) " x  y signif raw-count-rate corr-count-rate aperture radius extension flag"
!write(*,*) "-----------------------------"
!do i = 1, min(20, size(sortindex) )
!
!index = sortIndex(i) 
!write(*,*) "L607 index=", index, "source x,y=", source(index)%xpos, source(index)%ypos, &
!     " signif=", source(index)%significance, " raw=", source(index)%rawCountRate, &
!     " bkg=", source(index)%rawBackgroundCountRate, " extFlag=", source(index)%extensionFlag
!!     " corr=", source(index)%corrCountRate, " corr_bkg=", source(index)%corrBackgroundCountRate
!!write(*,*) " apertureRadius=", source(index)%apertureRadius, " extensionFlag=", &
!!     source(index)%extensionFlag
!enddo
!write(*,*) "-----------------------------"


!call SaveRealImage1(ccdimage, nx1, ny1, "ccdimage.fits", "test")




                   
    nullify(ccdImage, backgroundImage, qualityImage, sourceNumImage)     
 

    deallocate(sortIndex, stat = status)
    nullify(sortIndex)
    call CheckDeallocation(status, "sortIndex (Subroutine AperturePhotometry)")

    deallocate(sourceCoords, stat = status)
    nullify(sourceCoords)
    call CheckDeallocation(status, "sourceCoords (Subroutine AperturePhotometry)")

    deAllocate(backgroundPixel, stat = status)
    nullify(backgroundPixel)
  
    call CheckDeallocation(status, "backgroundPixel (Subroutine AperturePhotometry)")


    deAllocate(objectWidth, stat = status)
    call CheckDeallocation(status, "objectWidth (Subroutine AperturePhotometry)")
    nullify(objectWidth)











end subroutine AperturePhotometry2




!*********************************************
!
! Function GetApertureCounts
! Computes the counts within a given aperture
! Returns the value via the function name
!
!*********************************************
real(double) function GetApertureCounts(&
                    x0,  &  ! The x-pixel coordinate of the centre of the aperture
                    y0,  &  ! The y-pixel coordinate of the centre of the aperture
                radius)  ! The aperture radius (pixels) for summing the counts
               
    implicit none

    !*********************************************
    ! Input (unmodfied) variables
    !*********************************************
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: radius
    
    ! Local variables
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
   

    apertureRadius = radius        
           
    ! Determine upper and lower array indices for
    ! summing counts
   
    i1 = int(x0 - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(x0 + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(y0 - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(y0 + radius + 0.0001)
    j2 = min(ju, j2)
 
    GetApertureCounts = 0.0_double 

    apertureArea = 0.0_double

    ! Sum the pixels in the strip round the centre


!    write(*, *) "Getaperturecounts x0=", x0, " y0=", y0, " i1=", i1, " i2=", &
!                     i2, " j1=", j1, " j2=", j2


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   

    do i = i1, i2
        do j = j1, j2         
            GetApertureCounts = GetApertureCounts + PixelValue1(x0, y0, i, j)
            
!           write(*, *) "i=", i, " j=", j, " pixelvalue=", PixelValue1(x0, y0, i, j), &
!              " image=", ccdimage(i, j)
        
        end do
    end do
!write(*, *) "total aperture counts=", getaperturecounts, " aperturearea=", aperturearea
  
end function GetApertureCounts       
!*********************************************
! Function GetApertureCounts1
! Computes the counts within a given aperture
! Returns the value via the function name
! This version for extended sources only
!*********************************************
real(double) function GetApertureCounts1(&
                    x0,  &  ! The x-pixel coordinate of the centre of the aperture
                    y0,  &  ! The y-pixel coordinate of the centre of the aperture
                radius  )  ! The aperture radius (pixels) for summing the counts
             
    implicit none

    !*********************************************
    ! Input (unmodfied) variables
    !*********************************************
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: radius
   
    !********************************************
    ! Local variables
    !*******************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
   

    apertureRadius = radius        
        
    ! Determine upper and lower array indices for
    ! summing counts

    i1 = int(x0 - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(x0 + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(y0 - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(y0 + radius + 0.0001)
    j2 = min(ju, j2)
 
    GetApertureCounts1 = 0.0_double 

    apertureArea = 0.0_double




    !***************************************************
    ! Sum the pixels in the strip round the centre
    !***************************************************
     
  
    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 10000000.0_double 
    maxExposure = 0.0_double
   

    do i = i1, i2
        do j = j1, j2         
            GetApertureCounts1 = GetApertureCounts1 + PixelValue2(i, j)
            
       
  
        end do
    end do

  
end function GetApertureCounts1       
!********************************************
! Function PixelValue1
! Purpose - returns the value of the pixel
! at the given position weighted by the area
! of the aperture contained within the pixel
!*********************************************
real(double) function PixelValue1(x0, y0, iIndex, jIndex, areaOut)
    implicit none   
    real (single), intent(in) :: x0, y0
    integer (int32), intent(in) :: iIndex, jIndex
    real(double), intent(out), optional :: areaOut  
 
    real (single) :: r1, r2, r3, r4
    real (single) :: x, y, x1, x2, y1, y2
    real (single) :: minRadius, maxRadius
    real(double) :: area    
    real(single) :: fCounts
   
     
    x = real(iIndex)  
    y = real(jIndex) 
 
    x1 = x - x0
    y1 = y - y0
    x2 = x1 + 1.0
    y2 = y1 + 1.0

    r1 = sqrt(x1 * x1 + y1 * y1)
    minRadius = r1
    maxRadius = r1
 
    r2 = sqrt(x2 * x2 + y1 * y1)
    minRadius = min(minRadius, r2)
    maxRadius = max(maxRadius, r2)

    r3 = sqrt(x1 * x1 + y2 * y2)
    minRadius = min(minRadius, r3)
    maxRadius = max(maxRadius, r3)

    r4 = sqrt(x2 * x2 + y2 * y2)
    minRadius = min(minRadius, r4)
    maxRadius = max(maxRadius, r4)
  

    if (apertureRadius .le. minRadius) then
        PixelValue1 =  0.0_double

        if(present(areaOut) ) then
            areaOut = 0.0
        end if

        return 
    end if

    if( isNullValue(CCDImage(iIndex, jIndex)) ) then

        fCounts = 0.0
    else
        fCounts = CCDImage(iIndex, jIndex)


        !*********************************************************************
        ! Maybe using an exposure map- get the mean, min and max  exposure
        !********************************************************************  
        call getExposure(iIndex, jIndex)
  

        ! Check that the pixel is okay to use
        if (associated(qualityImage)) then    
            if (qualityImage(iIndex, jIndex)) then ! Bad pixel- set source quality flag
                call setQualityFlag    
            end if
        end if


    end if  

  
    if (apertureRadius  .ge. maxRadius) then
        apertureArea = apertureArea + 1.0_double      
        PixelValue1 = fCounts
        area = 1.0
    else
        area = EffectivePixelArea(x1, y1)  ! Pixel area (1 if full pixel)
        pixelValue1 = area * fCounts
        apertureArea = apertureArea + area          
    end if

    if(present(areaOut) ) then
        areaOut = area
    end if


 
end function PixelValue1
!********************************************
! Function PixelValue2
! Purpose - For extended sources
!*********************************************
real(double) function PixelValue2(iIndex, jIndex)
       
    
    integer(int32), intent(in) :: iIndex, jIndex
  
  
    ! Check that the pixel is okay to use
    if (associated(qualityImage)) then 
        if (qualityImage(iIndex, jIndex)) then ! Bad pixel- set source quality flag
            call setQualityFlag

        end if
    end if
    
    apertureArea = apertureArea + 1.0_double      

    if( isNullValue(CCDImage(iIndex, jIndex) ) ) then

        PixelValue2 = 0.0_double
    else
    
        PixelValue2 = CCDImage(iIndex, jIndex)
 

        !*********************************************************************
        ! Maybe using an exposure map- get the mean, min and max  exposure
        !********************************************************************  
        call getExposure(iIndex, jIndex)
  
   end if
 
end function PixelValue2
!**************************************************
! Function EffectivePixelArea
! Computes the area of a pixel contained
! within the aperture and returns the value
! via the name
! This will shortly be replaced by a better routine
!***************************************************
real(double) function EffectivePixelArea(x1, y1)
  
    real(single), intent(in) :: x1, y1
    real(single) :: radiusSqr
    real(single) :: xPos, yPos
    
    integer(int32) :: nIn
    integer(int32) :: i, j
     

    real(single), save :: deltaY(1 : 99)
    
    forall (j = 1 : 99) deltaY(j) = y1 + 0.01 * real(j)

    nIn = 0

    do i = 2, 100

        xPos = x1 + 0.01 * real(i - 1)
        do j = 1, 99     
            ypos = deltaY(j)        
            radiusSqr = xPos * xPos + yPos * yPos 
            if (sqrt(radiusSqr) .le. apertureRadius) then                         
                nIn = nIn + 1
            end if
        end do
    end do

    EffectivePixelArea = dble(nin) / 9801_double 

end function EffectivePixelArea


!***********************************************
! ComputeSourceRateError
! Computes the source counts error and returns 
! the value via the name
! NB sourceCounts is the background-subtracted counts
! and backgroundcounts the background counts
!***********************************************
real(double) function ComputeSourceRateError1(sourceCounts, backGroundCounts, exposure)
    real(double), intent(in) :: sourceCounts       ! The background-subtracted source counts
    real(double), intent(in) :: backGroundCounts   ! The background counts
    real(single), intent(in) :: exposure
  

    ComputeSourceRateError1 = sqrt(max(0.0_double, sourceCounts + 2.0_double * backGroundCounts)) / dble(exposure)
 
end function ComputeSourceRateError1




!******************************************************************
! Function GetMaximumAperture
! Finds the maximum aperture size that can be used for photomery
! Returns the value via the function name
! Also return "type" -
! type - 1 = no object within aperture radius pixels
!        2 = object within aperture radius, but outside minimum aperture radius
!        3 = object within minimum aperture radius
!*******************************************************************
real(single) function GetMaximumAperture(noSources, index, type)
    integer(int16), intent(in) :: noSources        ! The number of sources    
    integer(int32), intent(in) :: index            ! Index of source 
    integer(int32), intent(out) :: type            ! Determines what kind of photometry can be performed
    TYPE(OMPoint) :: centre                   ! Coords of source
    real(single) :: minimumSeparation  ! Minimum distance between source of interest and other sources
    real(single) :: borderDistance     ! Distance to the edge of the image from the source 
    real(single) :: okayDistance       ! Minimum allowed separation between source and nearest neighbour   
    real(single) :: distance           ! Distance bewteen source and neighbour
    real (single):: d, d1
!    real(single) :: maximumCounts
    integer(int32) :: k    
   
    centre = sourceCoords(index)

    okayDistance = 2.0 * apertureRadius  ! Okay for aperture photometry if nearest star is 2 aperture radii distant
    borderDistance = 1e8
    minimumSeparation = 1e6
!    maximumCounts = 0.0
    type = 0

    !******************************************
    ! Checks to see if object is near the
    ! edge of the image
    !******************************************
    if (centre%x - xLeft .lt. apertureRadius) then
        borderDistance = centre%x - xLeft
        type = 4
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
 

    else if  (xRight - centre%x .lt. apertureRadius) then
        type = 4
        borderDistance = xRight - centre%x
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
 
 
    end if

    if (centre%y - yBottom .lt. apertureRadius) then
        type = 4
        borderDistance = min(centre%y - yBottom, borderDistance)
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
 
 
    else  if (yTop - centre%y .lt. apertureRadius) then
        type = 4
        borderDistance = min(yTop - centre%y, borderDistance)
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
 
 
    end if


    !***************************************************
    ! Find minimum distance between current source 
    ! and sources to its left. NB sourceCoords contains
    ! the x-coordinates of the sources in increasing 
    ! order.
    !***************************************************
    if (index .gt. 1) then
       
        k = index
    
        do while (k .gt. 1)
            k = k - 1
            if (sourceCoords(k)%extension .eq. 1) cycle
                
                           
            d = centre%x - sourceCoords(k)%x
            d = max(0.0, d)
            if (d .gt. apertureRadius + objectWidth(k) ) exit
            if (sourceCoords(k)%y .ge. centre%y) then
                d1 = sourceCoords(k)%y - centre%y
                d1 = max(0.0, d1)
                
            else
                d1 = centre%y - sourceCoords(k)%y
                d1 = max(0.0, d1)
            end if


            distance = sqrt(d * d + d1 * d1)
            
            if (distance + apertureRadius - objectWidth(k) .lt. minimumSeparation) then 

                minimumSeparation = distance + apertureRadius - objectWidth(k)
!                maximumCounts = max(maximumCounts, sourceCoords(k)%maxCounts)

            end if
  
        end do
    end if

    !***************************************************
    ! Find minimum distance between current source
    ! and sources to its right.
    !**************************************************
    if (index .lt. noSources) then
        k = index
        do while (k .lt. noSources)
            k = k + 1
            if (sourceCoords(k)%extension .eq. 1) cycle
           
            d = sourceCoords(k)%x - centre%x
            d = max(0.0, d)

            if (d .gt. apertureRadius + objectWidth(k)) exit

            if (sourceCoords(k)%y .ge. centre%y) then
                d1 = sourceCoords(k)%y - centre%y
                d1 = max(0.0, d1)
            else
                d1 = centre%y - sourceCoords(k)%y
                d1 = max(0.0, d1)
            end if

            distance = sqrt(d * d + d1 * d1)
            if (distance + apertureRadius - objectWidth(k) .lt. minimumSeparation) then
                minimumSeparation = distance + apertureRadius - objectWidth(k)
!                maximumCounts = max(maximumCounts, sourceCoords(k)%maxCounts)


            end if
            
        end do
    end if

!if (type .eq. 4) then
!write(*, *) "Line 801 type=4, minimum separation=", minimumSeparation, " border distance=", borderDistance
!end if

    if (type .eq. 4) then
      
        if (minimumSeparation .ge. borderDistance) then
            GetMaximumAperture = max( real(4 / binx), borderDistance)
            return
         end if
    end if    

   

    !****************************************************************
    !
    ! Having reached here, minimumSeparation is the minimum
    ! distance between the current source and its nearest neighbour
    ! apertureRadius is the default aperture radius (12/binx) pixels 
    ! minimumApertureRadius is the minimum-allowed aperture radius
    !
    !
    !****************************************************************


    if (minimumSeparation .ge. okayDistance) then
    
        type = 1
        currentSource%confusionFlag = 0
 
        GetMaximumAperture = apertureRadius

    else if (minimumSeparation .ge. 2.0 * minimumApertureRadius) then  
        type = 2 

!        if (maximumCounts .gt. 0.01 * centre%maxCounts) then

            GetMaximumAperture = 0.5 * minimumSeparation   ! - apertureRadius



!        else
!            GetMaximumAperture = apertureRadius
!        end if 
    
 
        currentSource%confusionFlag = ibset(currentSource%confusionFlag, 0 )
 
    else
        type = 3    ! Can't do photometry on
        GetMaximumAperture = 0.0
        currentSource%confusionFlag = ibset(currentSource%confusionFlag, 1 )
 
    end if


end function GetMaximumAperture


!*********************************************
!
! subroutineGetExtendedSourceCounts
! Computes the counts for an extended source
! Returns the value via the function name
! Uses the pixels assigned to the source
! on the SOURCENUM image.
!
!*********************************************
subroutine GetExtendedSourceCounts1(source, radius)

    ! Input (unmodfied) variables
    real(single), intent(in) :: radius 
    type(SourceParameters), intent(inout) :: source

    ! Local variables
    integer(int32) :: i, j, count
  
    real(single) :: x0, y0
      
    real(double) :: rawCounts, correctedCounts, countRate
    real(double) :: f
    real(double) :: backgroundRate, totalRate
    real(double) :: backgroundCounts, correctedBackgroundCounts 
    real(double) :: backgroundCountsPerPixel, backgroundRatePerPixel
    real(double) :: correction
    logical :: useBackgroundMap = .false.
    real(double) :: backgroundMode, meanExposure = 0.0_double
    integer ::  numExposurePixels = 0
!    integer, save :: count1 = 0    



 
           
!write(*, *) "**********************************************************"
!write(*, *) "GETEXTENDEDSOURCECOUNTS1"

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 10000000.0_double 
    maxExposure = 0.0_double
    

    !*****************************************************************
    !
    ! Determine the corrected background count-rate per pixel
    !
    !*****************************************************************
    !write(*,*) "L1245 extFlag=",source%extensionFlag, " raw=", source%rawCountRate
    !write(*,*) "L1246 raw-background=",source%rawBackgroundCountRate
 
    if (source%rawBackgroundCountRate > 0.0) then       ! Use the user-specified value

        backgroundMode = source%rawBackgroundCountRate  ! Use user background value
        backgroundRate = backgroundMode * PI * 144.0 / (dble(binX * binX) * source%exposure)


        ! The coincidence-loss correction for each background pixel
        correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
        backgroundCountsPerPixel = backgroundCountsMode !* correction
        backgroundRatePerPixel = backgroundCountsPerPixel / source%exposure
              

    else if (associated(backGroundImage)) then          ! Using a background image
        useBackgroundMap = .true.
        backgroundCounts = 0.0_double
        correctedBackgroundCounts = 0.0_double

    else 
       
        backgroundMode = backgroundCountsMode           ! Use global image mode                 
        backgroundRate = backgroundMode * PI * 144.0 / (dble(binX * binX) * source%exposure)
        !write(*,*) "L1266 backgroundMode=", backgroundMode, " backgroundrate=", backgroundRate
   


        ! The coincidence-loss correction for each background pixel
        correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
        backgroundCountsPerPixel = backgroundCountsMode * correction
        backgroundRatePerPixel = backgroundCountsPerPixel / source%exposure
      
    end if

  

    !***********************************************************************
    !
    !
    !
    !***********************************************************************

    count = 0 ! The number of pixels
    rawCounts = 0.0_double 
    correctedCounts = 0.0_double

    numExposurePixels = 0
    meanExposure = 0.0_double
    if( associated( exposureImage ) ) then
       !write(*,*) "L1295 (associated exposure Image)" 
       source%exposure = exposureConstant
    end if

    !write(*,*) "L1299 iMin=", source%iMin, " imax=", source%iMax, &
    !     " jMin=", source%jMin, " jmax=", source%jMax
    do i = source%iMin, source%iMax
       x0 = real(i) + 0.5
     
       do j = source%jMin, source%jMax
          !if (i .eq. source%iMin .and. j .eq. source%jMin) then
          !   write(*,*) "L1305 source Number=", source%number, " srcN image=",sourceNumImage(i, j) 
          !endif
          if (sourceNumImage(i, j) .eq. source%number) then ! The pixel is part of the sources

             if (associated(exposureImage )) then
                numExposurePixels = numExposurePixels + 1
                
                meanExposure = meanExposure + dble(exposureImage(i, j))
             end if
             y0 = real(j) + 0.5

             !****************************************************************
             !
             ! Compute the coincidence-loss correction for this pixel
             !
             !****************************************************************
             countRate= GetApertureCounts1(x0, y0, radius) / source%exposure                   
 
             f =  PixelValue2(i, j)
             if (i .eq. source%iMin .and. j .eq. source%jMin) then
                write(*,*) "L1325 countRate=", countRate, " f=", f
             endif
              
              
             rawCounts = rawCounts + f
             correctedCounts = correctedCounts + f * &
                  CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
             
             count = count + 1   
             
             if (useBackgroundMap) then       ! Use the background map
                
                backgroundCounts = backgroundCounts + backgroundImage(i, j)
                
                
             end if
          else
             !***************************************************************
             ! Added 2010-10-17 (vny) as the above piece of the code fails to
             ! to produce extended source counts
             !***************************************************************
             if (associated(exposureImage )) then
                !write(*,*) "L1350 using associated exposure image"
                numExposurePixels = numExposurePixels + 1

                meanExposure = meanExposure + dble(exposureImage(i, j))
             end if
             y0 = real(j) + 0.5

             !****************************************************************
             !
             ! Compute the coincidence-loss correction for this pixel
             !
             !****************************************************************
             countRate= GetApertureCounts1(x0, y0, radius) / source%exposure                   

             f =  PixelValue2(i, j)
             if (i .eq. source%iMin .and. j .eq. source%jMin) then
                !write(*,*) "L1364 countRate=", countRate, " f=", f
             endif
              
              
             rawCounts = rawCounts + f
             correctedCounts = correctedCounts + f * &
                  CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
               
             count = count + 1   

             if (useBackgroundMap) then       ! Use the background map
                !write(*,*) "L1375 using the background map"
                backgroundCounts = backgroundCounts + backgroundImage(i, j)
             end if
          end if

        end do
    end do


         


!write(*, *) "L1340 count=", count, " rawcounts=", rawCounts, " backgroundcounts=", backgroundCounts   

    if( associated(exposureImage) ) then

        meanExposure = meanExposure / dble(numExposurePixels)
!write(*, *) "L1345 Meanexposure=", meanExposure
     
        correction = meanExposure / exposureConstant
            
        backGroundCounts = backgroundCounts * correction
        rawCounts = rawCounts * correction
        correctedCounts = correctedCounts * correction 
        backgroundCountsMode = backgroundCountsMode * correction
        backgroundCountsPerPixel = backgroundCountsPerPixel * correction
        source%exposure = meanExposure
    end if

    

    if (useBackgroundMap) then            


        f = backgroundCounts / dble(count) 
        backgroundRate = f * PI * radius * radius / source%exposure
        

!write(*, *) "L1413 backgroundrate=", backgroundrate, " exposure=", source%exposure

        correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
!write(*, *) "L1416 correction=", correction

        backgroundCountsPerPixel = f * correction
        backgroundRatePerPixel = backgroundCountsPerPixel / source%exposure
        rawCounts = rawCounts - backgroundCounts


!write(*, *) "L1423 rawcounts=", rawcounts, " backgroundcounts=", backgroundCounts

        correctedCounts = correctedCounts - backgroundCounts * correction

    else
        rawCounts = rawCounts - dble(count) *  backgroundCountsMode
        correctedCounts = correctedCounts - dble(count) * backgroundCountsPerPixel
        backgroundCounts = backgroundCountsMode * dble(count)
     
 

    end if


    ! The raw count rates
    source%rawCountRate = rawCounts / source%exposure
    source%rawCountRateErr = ComputeSourceRateError(rawCounts, backGroundCounts, source%exposure)
    source%rawBackGroundCountRate = backgroundCounts / source%exposure 
    source%corrBackGroundCountRate = backgroundRatePerPixel * dble(count)

    source%corrCountRate = correctedCounts / source%exposure
    source%corrCountRateErr =  ComputeSourceRateError(correctedCounts, backGroundCounts, source%exposure)
                   

    !**********************************************************************
    !
    ! The source coincidence-loss correction
    !
    !**********************************************************************
    totalRate = source%rawCountRate + source%rawBackgroundCountRate
    if (totalrate .gt. 0.0) then
        source%rightAscension =  (source%corrCountRate + source%corrBackgroundCountRate) / &
           totalrate
    else
        source%rightAscension = 1.0
    end if
    ! The background coincidence-loss correction
    if (source%rawBackgroundCountRate .gt. 0.0) then
        source%declination =  source%corrBackGroundCountRate / source%rawBackGroundCountRate
    else  
        source%declination = 1.0

    end if


    !*****************************************
    !
    ! Recalculate the source significance
    !
    !*****************************************   
    source%Significance = ComputeSourceSignificance(real(rawCounts), real(backgroundCounts))   
    source%apertureRadius = nullValue  
    source%psf1Correction = 0.0
    source%coincidenceLossCorr = max(0.0, (source%corrCountRate - source%rawCountRate) * 100.0 / source%rawCountRate) 
    if(associated( exposureImage ) ) then 
        source%exposure = meanExposure
    end if  


!write(*, *) "L1482 rawcounts=", rawCounts, " backgroundcounts=", backgroundCounts
!write(*, *) "L1483 rawcountrate=", source%rawCountRate, " backgroundcountrate=", source%rawbackgroundCountRate
!write(*, *) "L1484 correctedcountrate=", source%corrCountRate, " backgroundcountrate=", source%corrbackgroundCountRate



end subroutine GetExtendedSourceCounts1       
!*********************************************
! subroutine GetExtendedSourceCounts2
! For extended sources with elliptical apertures
! Computes the count-rates/etc for an extended source
! 
!*********************************************
subroutine GetExtendedSourceCounts2(source, coincLossRadius, semiA, semiB, angle)

    ! Input (unmodfied) variables
    real(single), intent(in) :: coincLossRadius, semiA, semiB, angle
    type(SourceParameters), intent(inout) :: source

    ! Local variables
    integer(int32) :: i, j
    real(single) :: x0, y0, theta
      
    real(double) :: f, rawCounts, correctedCounts, countRate
    real(double) :: backgroundRate
    real(double) :: backgroundCounts = 0.0_double, correctedBackgroundCounts
    real(double) :: correction, pixArea, totalArea
    real(single) :: error    
    logical :: useBackgroundMap = .false.
    logical :: useImage = .false.
!integer, save :: count = 0
 
    ! Determine the corrected background count-rate per pixel using 
    if (angle .lt. 0.0) then
        theta = angle + 180.0
    else
        theta = angle
    end if

!write(*, *) "GetextendedSourceCounts2 line 993, exposure=", source%exposure
    ! the elliptical background annulus


    if (source%rawBackgroundCountRate > 0.0) then ! Using a fixed background mode to get background counts
        backgroundCounts = source%rawBackgroundCountRate * PI * semiA * semiB
      
    else if (associated(backGroundImage)) then ! Using a background image
        useBackgroundMap = .true.

    else ! Using the mode from the image
!if (count .lt. 5) then
!write(*, *) "calling getbackgroundmode2"
!end if
        
        backgroundCounts = GetBackGroundMode2( &
                     source%xPos,              &   ! The centre x-pixel coordinate of the aperture
                     source%yPos,              &   ! The centre y-pixel corrdinate of the aperture 
                     source%innerBckRadius,    &   ! The aperture inner radius
                     source%outerBckRadius,    &   ! The aperture outer radius
                     source%axialRatio,        &   ! The ellipse axial ratio
                     theta,                    &   ! The ellipse position angle
                     error)      ! The error
        useImage = .true.

    end if    


! if (count .lt. 5) then
!if (usebackgroundmap) write(*, *) "using background map"
!write(*, *) "Line 1130 backgroundcounts=",   backgroundCounts, " xpos=", source%xpos, " ypos=", source%ypos, &
! " inner bck=", source%innerBckRadius, " outer=",   source%outerBckRadius, " axial ratio=", source%axialratio 
!end if           


 
    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
    
    rawCounts = 0.0_double
    correctedCounts = 0.0_double
    totalArea = 0.0_double
  
  
    do i = max(0, int(floor(source%xPos) - ceiling(semiA)) - 1 ),  &
                min(nx - 1, int(ceiling(source%xPos) + semiA + 0.1) + 1) 
        x0 = real(i) + 0.5
     
        do j = max(0, int(floor(source%yPos) - ceiling(semiA) ) - 1),  &
                     min(ny - 1, int(ceiling(source%yPos) + semiA + 0.1) + 1) 
       
                            
            y0 = real(j) + 0.5 ! 
            pixArea = PixelArea(x0 + 0.5, y0 + 0.5, source%xPos, source%yPos, semiA, semiB, theta)

            if (pixArea <= 0.0) cycle
            totalArea = totalArea + pixArea
            ! Compute the coincidence-loss correction for this pixel
            countRate= GetApertureCounts1(x0, y0, coincLossRadius) / source%exposure 
            if (pixArea .lt. 1.0) then                  
                f =  PixelValue2(i, j) * pixArea            

                if (useBackgroundMap) &                  
                    backgroundCounts = backgroundCounts + backgroundImage(i, j) * pixArea
         
            else
                f =  PixelValue2(i, j)            

                if (useBackgroundMap) &                  
                    backgroundCounts = backgroundCounts + backgroundImage(i, j)
         
            end if

            rawCounts = rawCounts + f
            correctedCounts = correctedCounts + f * &
                        CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
                                 
        end do
    end do
!write(*, *) "Line 1191 background counts=", backgroundcounts

    if (useImage) backgroundCounts = backgroundCounts * totalArea

!write(*, *) "Line 1195 background counts=", backgroundcounts












!if (count .lt. 5) then
!count = count + 1
!write(*, *) "rawCounts=", rawCounts, " background counts=", backgroundCounts, " total area=", totalArea, &
!        " semia=", semia, " semib=", semib, " theoretical area=", (PI * semia * semib)
!if (associated(backgroundimage)) write(*, *) "using a background image"
!end if

    backgroundRate = backgroundCounts / source%exposure ! Background count rate
    source%rawBackgroundCountRate = backgroundCounts / source%exposure
      





    !**********************************************************
    ! The coincidence-loss correction for the background
    !*********************************************************
    correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
    source%corrBackGroundCountRate = backgroundRate * correction ! Corrected background rate



!write(*, *) "Line 1229 correction=", correction

    correctedBackgroundCounts = backgroundCounts * correction

   
   

    ! Background-subtracted counts
    rawCounts = rawCounts - backgroundCounts
    source%rawCountRate = rawCounts / source%exposure

    !*****************************************
    ! Recalculate the source significance
    !*****************************************   
    source%Significance = ComputeSourceSignificance(real(rawCounts), real(backgroundCounts))

 


    source%rawCountRateErr = ComputeSourceRateError(rawCounts, backGroundCounts, source%exposure)
               
         
!write(*, *) "background subtracted rawCount rate=", source%rawCountRate, " background rate=", source%rawBackgroundCountRate

    ! Corrected background-subtracted counts
    source%corrCountRate = (correctedCounts - correctedBackgroundCounts) / source%exposure
    source%corrCountRateErr =  ComputeSourceRateError(correctedCounts, correctedBackGroundCounts, source%exposure) 


!write(*, *) "GetExtendedsourcecounts2 line 1254 corrected counts=", correctedcounts, &
!" correctedbackgroundcounts=", correctedbackgroundcounts


           
   
!    source%apertureRadius = nullValue  
    source%psf1Correction = 0.0



    ! The source coincidence-loss correction
    if (source%rawCountRate .gt. 0.0) then
        source%rightAscension =  source%corrCountRate / source%rawCountRate
    end if
    ! The background coincidence-loss correction
    if (source%rawBackgroundCountRate .gt. 0.0) then
        source%declination =  source%corrBackGroundCountRate / source%rawBackGroundCountRate
    end if



    if (source%rawCountRate .gt. 0.0) then
        source%coincidenceLossCorr = source%corrCountRate / source%rawCountRate 
    else
       source%coincidenceLossCorr = 1.0
    end if
  



!    write(*, *) "Extended source number ", sourceNumber, " x=", source%xsource, " y=", source%ysource 
!    write(*, *) " Original count rate=", source%rawCountRate, " new rate=" , source%correctedCountRate
!    write(*, *) " Original background rate=", source%rawBackgroundCountRate, &
!                " new rate=" , source%correctedBackgroundCountRate, " radius=",  source%apertureRadius
!    write(*, *) "Significance=",  source%Significance

end subroutine GetExtendedSourceCounts2      
!*********************************************
! Function GetBackGroundMode1
! For use with circular annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation)
! 
!*********************************************
real (double) function GetBackGroundMode1( &
    levelImage,   &
    x0,           &   ! The centre x-pixel coordinate of the aperture
    y0,           &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius1, &   ! The aperture inner radius
    outerRadius1, &   ! The aperture outer radius
    error)            ! The error


    implicit none
    
    ! Input variables
    integer(int16), pointer :: levelImage(:, :)  
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius1
    real(single), intent(in) :: outerRadius1
    real(single), intent(out) :: error
  
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: x, y, dx, dy, r, radius, innerRadius, outerRadius
   
   
    ! Scan the annulus region and add pixel values to the 
    ! backGroundPixel array

!    if ( btest(currentSource%qualityFlag, 4) ) then
!        innerRadius = currentSource%axialRatio
!        outerRadius = innerRadius + real(4 / binx)
!    else
        innerRadius = innerRadius1
        
        outerRadius = outerRadius1
      
!    end if 
 

    i0 = int(x0)
    j0 = int(y0)
    nData = 0  

  
!write(*, *) "Getbackgroundmode1 line 1731 x=", x0, " y=", y0, " inner radius=", innerRadius, " outer=", outerRadius    
!write(*, *) "minval(levelimage)=", minval(levelImage), " max=", maxval(levelImage)



    r = ceiling(outerRadius)

    i1 = i0 - r - 1
    i1 = max(il, i1)
  
    j1 = j0 - r - 1
    j1 = max(jl, j1)

  
    ! Determine upper and lower array indices for
    ! summing counts
    r = ceiling(outerRadius)

        
    i2 = i0 + r + 1
    i2 = min(iu, i2)  

  
    j2 = j0 + r + 1
    j2 = min(ju, j2)
 
    
iLoop : do i = i1, i2

        x = real(i) + 0.5  
        dx = x - x0
        do j = j1, j2          
               
            if (associated(qualityImage)) then
                if (qualityImage(i, j)) cycle
            end if



            if ( levelImage(i, j) .ne. 0) then
                cycle
            end if
            
           
            y = real(j) + 0.5
            dy = y - y0
            radius = sqrt(dx * dx + dy * dy)
            if (radius .ge. innerRadius .and. radius .le. outerRadius) then         
                if ( .not. isNullValue( CCDImage(i, j) ) ) then              
                    nData = nData + 1 
                    backGroundPixel(nData) = CCDImage(i, j)

!write(*, *) nData, ") i=", i, " j=", j, " pixel=", ccdImage(i, j), " levelimage=", levelImage(i, j)  
                    if (nData == maxNumPixels) then

                        exit iLoop
                    end if 
                end if 
            end if
            
                             
        end do 

    end do iLoop

!write(*, *) "Line 1794 ndata=", nData, " mindata=", minData

    ! Compute the median background pixel value

    if (nData .ge. minData) then  ! 

        GetBackgroundMode1 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)

    else if (associated(backgroundImage)) then  ! Use the background image



        nData = 0
iLoop1 :    do i = i1, i2

            x = real(i)  
            dx = x - x0
            do j = j1, j2          
              
            
                y = real(j)
                dy = y - y0
                r = sqrt(dx * dx + dy * dy)
                if (r .ge. innerRadius .and. r .le. outerRadius) then         
              
                    nData = nData + 1 
                    backGroundPixel(nData) = backgroundImage(i, j)  
                    if (nData == maxNumPixels) then

                        exit iLoop1
                    end if  
                end if                                        
            end do
        end do iLoop1


        
        if(ndata .ge. minData) then
            GetBackgroundMode1 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
        else
          
            GetBackgroundMode1 = backgroundCountsMode
            error = backgroundCountsModeError
        end if 


    else
        GetBackgroundMode1 = backgroundCountsMode
        error = backgroundCountsModeError

    end if



!if (abs(x0 - 945) < 5.0 .and. abs(y0 - 789.0) < 5.0) then
!write(*, *) "Source x=", x0, " y=", y0, " the final background pixels"
!do i = 1, ndata
!write(*, *) i, " ) ", backgroundpixel(i)
!end do
!end if

!write(*, *) "GetBackgroundMode1 line 1858 innerRadius=", innerRadius, " outerradius=", outerRadius, &
!  " getbackgroundmode1=", getBackgroundMode1
 


end function GetBackgroundMode1       


!*********************************************
! Function GetBackGroundMode2
! For use with elliptical annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation) 
!*********************************************
real (double) function GetBackGroundMode2( &

    x0,          &   ! The centre x-pixel coordinate of the aperture
    y0,          &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius, &   ! The aperture inner radius
    outerRadius, &   ! The aperture outer radius
    axialRatio,  &   ! The ellipse axial ratio
    theta1,       &   ! The ellipse position angle
    error)      ! The error

    implicit none
    
    ! Input variables  
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius
    real(single), intent(in) :: outerRadius
    real(single), intent(in) :: axialRatio
    real(single), intent(in) :: theta1
    real(single), intent(out) :: error
  
    real(single) r, semiA1, semiA2, semiB1, semiB2, theta 

  
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: x, y

!integer, save :: count = 0

    semiA1 = innerRadius * axialRatio
    semiA2 = outerRadius * axialRatio
    semiB1 = innerRadius
    semiB2 = outerRadius

    theta = theta1 * degreesToRadians
    ! Scan the annulus region and add pixel values to the 
    ! backGroundPixel array

    i0 = int(x0)
    j0 = int(y0)
    nData = 0  
        
    i0 = int(x0)
    j0 = int(y0)
       
    r = ceiling(outerRadius * axialRatio)
   
    ! Determine upper and lower array indices for
    ! summing counts

    i1 = i0 - r - 1
    i1 = max(il, i1)
  
    i2 = i0 + r + 1
    i2 = min(iu, i2)  

    j1 = j0 - r - 1
    j1 = max(jl, j1)

    j2 = j0 + r + 1
    j2 = min(ju, j2)
 

iLoop :   do i = i1, i2

        x = real(i)  
   
        do j = j1, j2          
                    
            if (associated(qualityImage)) then
                if (qualityImage(i, j)) cycle
            end if

            if (associated(sourceNumImage)) then 
                if (sourceNumImage(i, j) .gt. 0) cycle
            end if 

           
            y = real(j)
              
            !********************************************************************
            ! Point has to be within the inner and outer ellipse
            !********************************************************************
            if (.not. PointInEllipse(x, y, x0, y0, semiA1, semiB1, theta) .and. &
                      PointInEllipse(x, y, x0, y0, semiA2, semiB2, theta)) then
          

                if ( .not. isNullValue( CCDImage(i, j) ) ) then              
                
                    nData = nData + 1 
                    backGroundPixel(nData) = CCDImage(i, j)  
 !                  ccdimage(i, j) = -1.0
                    if (nData == maxNumPixels) then
                        exit iLoop
                    end if
                end if  
            end if
                    
            
        end do
    end do iLoop

!if (count .lt. 5) then
!count = count + 1
!write(*, *) "getbackgroundmode2  semia1=", semia1, " semia2=", semia2, " semib1=", semib1, " semib2=", semib2, &
!" maxnumpixels=", maxnumpixels, " ndata=", ndata, " global background mode=", backgroundCountsMode
!end if

    ! Compute the median background pixel value
    
    if (nData .gt. 100) then   


        GetBackgroundMode2 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)





!if (count .lt. 5) then
!write(*, *) "ndata=", ndata, " mode=",   GetBackgroundMode2 
!end if




    else


!write(*, *) "Usingbackgroundcountsmode of ",  backgroundCountsMode


        GetBackgroundMode2 = backgroundCountsMode
        error = backgroundCountsModeError



!if (count .lt. 5) then
!write(*, *) "getbackgroundmode2 Line 1981 ndata=", ndata, " mode=",   GetBackgroundMode2 
!end if


        
    end if

 
end function GetBackgroundMode2       

!*********************************************
!
! Function GetBackGroundMode3
! For use with circular annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation) 
!
!*********************************************
real (double) function GetBackGroundMode3( &
    x0,          &   ! The centre x-pixel coordinate of the aperture
    y0,          &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius1, &   ! The aperture inner radius
    outerRadius1, &   ! The aperture outer radius

    error)      ! The error

    implicit none
    
    ! Input variables  
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius1
    real(single), intent(in) :: outerRadius1
    real(single), intent(out) :: error
  
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: innerRadius, outerRadius, x, y, dx, dy, r, radius
    integer :: iter
     ! Scan the annulus region and add pixel values to the 
    ! backGroundPixel array

    innerRadius = innerRadius1
    outerRadius = outerRadius1

    i0 = int(x0)
    j0 = int(y0)
    nData = 0  

  
!write(*, *) "Line 2071 Get background mode 3 inner radius=", innerRadius, " outer=", outerRadius    




    r = ceiling(outerRadius)

    i1 = i0 - r - 1
    i1 = max(il, i1)
  
    j1 = j0 - r - 1
    j1 = max(jl, j1)



    do iter = 0, 2        
     
       
        outerRadius = outerRadius1 + real(iter * 2)
   

        ! Determine upper and lower array indices for
        ! summing counts
        r = ceiling(outerRadius)

        
        i2 = i0 + r + 1
        i2 = min(iu, i2)  

  
        j2 = j0 + r + 1
        j2 = min(ju, j2)
 
    
iLoop : do i = i1, i2

            x = real(i) + 0.5  
            dx = x - x0
            do j = j1, j2          
               
                if (associated(qualityImage)) then
                    if (qualityImage(i, j)) cycle
                end if

           
                y = real(j) + 0.5
                dy = y - y0
                radius = sqrt(dx * dx + dy * dy)
                if (radius .ge. innerRadius .and. radius .le. outerRadius) then         
                    if ( .not. isNullValue( CCDImage(i, j) ) ) then              
                  
                        nData = nData + 1 
                        backGroundPixel(nData) = CCDImage(i, j)  
                        if (nData == maxNumPixels) then

                            exit iLoop
                        end if  
                    end if
                end if
                              
            end do
        end do iLoop


        if (nData .gt. minData) exit 
    
        innerRadius = outerRadius
    end do

!write(*, *) "Line 2140, ndata=", ndata, " mindata=", minData

    ! Compute the median background pixel value
    if (nData .gt. minData) then  ! 

        !GetBackgroundMode3 = BackgroundMode(nData, backGroundPixel, backgroundSigma, error)
        GetBackgroundMode3 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)




    else if (associated(backgroundImage)) then  ! Use the background image

        nData = 0
iLoop1 :    do i = i1, i2

            x = real(i)  
            dx = x - x0
            do j = j1, j2          
              
            
                y = real(j)
                dy = y - y0
                r = sqrt(dx * dx + dy * dy)
                if (r .ge. innerRadius1 .and. r .le. outerRadius1) then         
              
                    nData = nData + 1 
                    backGroundPixel(nData) = backgroundImage(i, j)  
                    if (nData == maxNumPixels) then

                        exit iLoop1
                    end if  
                end if                                        
            end do
        end do iLoop1


!        GetBackgroundMode3 = BackgroundMode(nData, backGroundPixel, backgroundSigma, error)

        GetBackgroundMode3 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
    end if

!write(*, *) "Line 2182 getbackgroundmode3=", getBackgroundMode3
end function GetBackgroundMode3       




!*********************************************
! Function GetBackGroundMode3
! For use with circular annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation) 
!*********************************************
real (double) function GetBackGroundMode4(         &
    image,                                         &
    x0,                                            &   ! The centre x-pixel coordinate of the aperture
    y0,                                            &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius,                                   &   ! The aperture inner radius
    outerRadius,                                   &   ! The aperture outer radius
    error)                                             ! The error

    implicit none
    
    ! Input variables  
    real(single), pointer :: image(:, :) 
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius
    real(single), intent(in) :: outerRadius
    real(single), intent(out) :: error
  
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: x, y, dx, dy, r, radius
    
  



    i0 = int(x0)
    j0 = int(y0)
    nData = 0  

  

    r = ceiling(innerRadius)

    i1 = i0 - r - 1
    i1 = max(lbound(image, 1), i1)
  
    j1 = j0 - r - 1
    j1 = max(lbound(image, 2), j1)



     
    ! Determine upper and lower array indices for
    ! summing counts
   

        
    i2 = i0 + r + 1
    i2 = min(i2, ubound(image, 1))  

  
    j2 = j0 + r + 1
    j2 = min(j2, ubound(image, 2))
 
    nData = 0    
    do i = i1, i2

        x = real(i) + 0.5  
        dx = x - x0
        do j = j1, j2          
                           
            y = real(j) + 0.5
            dy = y - y0
            radius = sqrt(dx * dx + dy * dy)
            if (radius .ge. innerRadius .and. radius .le. outerRadius) then         

                if ( .not. isNullValue( image(i, j) ) ) then              
             
                    nData = nData + 1 
                    backGroundPixel(nData) = image(i, j)  
                    if (nData == maxNumPixels) then

                        exit 
                    end if  
                 end if   
             end if  
                              
        end do
    end do


!write(*, *) "Getbackgroudnmode4 ndata=", ndata, " mindata=", mindata, " x0=", x0, " y0=", y0

    ! Compute the median background pixel value
    if (nData .gt. minData) then  ! 

        
        GetBackgroundMode4 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)


    else
        GetBackgroundMode4 = backgroundCountsMode

    end if

end function GetBackgroundMode4       





!*******************************************************
!
! Subroutine Initialise
!
!******************************************************
subroutine Initialise(noSources, nx1, ny1)
    integer(int16), intent(in) :: noSources, nx1, ny1

 
    real(single) :: calRadius
    integer(int32) :: status
    
    nx = nx1
    ny = ny1
  
    coincLossRadius = real(12 / binX) 
    ! Set up the background annulus
    
    calRadius =  CAL_omGetApertureRadius() / real(binx) 
!   write(*, *) "cal radius=" , calradius

    apertureRadius = real(12 / binx)
    innerRadius = calRadius + 2.0
    
    outerRadius = sqrt(areaRatio * apertureRadius * apertureRadius + innerRadius * innerRadius)

!write(*, *) "Line 1553 apertureRadius=", apertureRadius, " innerRadius=", innerRadius, " outerRadius=", outerRadius, &
!     " calradius=", calRadius
    call InitialiseArrays
 

    sortIndex => AllocateInt32(1, int(noSources, int32), "sortIndex (AperturePhotometry)")
     
    allocate(sourceCoords(1 : int(noSources, int32)), stat = status)
    call CheckAllocation(status, "sourceCoords (AperturePhotometry)")
 
    objectWidth => AllocateSingle(1, int(noSources, int32), "objectWidth (AperturePhotometry)")

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

end subroutine Initialise

!********************************************************
!
! Should be called at the very beginning of photometry
!
!*******************************************************
subroutine InitialiseArrays
  

    maxNumPixels = 30000
    if (associated(backgroundPixel)) return
  
    
    backgroundPixel => AllocateSingle(1, maxNumPixels, "backgroundPixel (AperturePhotometry)")


 
end subroutine InitialiseArrays
!********************************************************
!
! Should be called when finished doing photometry
!
!*******************************************************
subroutine deleteArrays

    if (associated(backgroundPixel)) then
        deallocate(backgroundPixel)
        nullify(backgroundPixel)
    end if


end subroutine DeleteArrays


!*************************************************************
!
! Point-source photometry
! Computes aperture photometry using the input radius
!
!*************************************************************
subroutine PointSourcePhotometry1(source, radius) 
    type(SourceParameters), intent(inout) :: source
    real(single), intent(in) ::radius
    real(double) :: sourceCounts, backgroundCounts
    real(single) :: backgroundCountsError
    
    real(double) :: backGroundCountsPerPixel
    real(single) r1, r2, area

    apertureRadius = radius

!write(*, *) "PointSourcePhotometry1 radius=", radius, " x=", source%xpos, " y=", source%ypos

    area = PI * radius * radius ! The required aperture area




    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 100000000.0_double 
    maxExposure = 0.0_double
    
    if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
        r1 = source%innerBckRadius
        r2 = source%outerBckRadius
    else 
        r1 =  innerRadius
        r2 =  outerRadius
    end if

    ! Get the total counts within the aperture   


    sourceCounts = GetApertureCounts(source%xPos - 0.5, source%yPos - 0.5, radius)
 
    !***********************************************************
    ! Get the background   
    !***********************************************************
    if (source%rawBackgroundCountRate .gt. -0.00001) then
        backgroundCountsPerPixel = source%rawBackgroundCountRate


    else if(associated(backgroundImage)) then ! Use the background image
        
        backGroundCountsPerPixel = GetBackGroundMode4(backgroundImage, source%xPos - 0.5, source%yPos - 0.5, &
                              r1, r2, backGroundCountsError)

    else if (associated(sourceNumImage)) then

        backGroundCountsPerPixel = GetBackGroundMode1(sourcenumImage, source%xPos - 0.5, source%yPos - 0.5, &
                              r1, r2, backGroundCountsError)

!write(*, *) "Line 2441 backgroundcountsperpixel=", backgroundCountsPerPixel, " r1=", r1, " r2=", r2
    else   

        backGroundCountsPerPixel = GetBackGroundMode3(source%xPos - 0.5, source%yPos - 0.5, &
                              r1, r2, backGroundCountsError)

    end if



    backGroundCounts = backGroundCountsPerPixel * apertureArea
    backGroundCountsError = backGroundCountsError * apertureArea 
  

       
  
    source%rawBackGroundCountRate = backGroundCounts / source%exposure
    source%rawBackGroundCountRateErr = sqrt(backgroundcounts) / source%exposure

  

    ! Subtract the background and convert to count-rate
    sourceCounts = sourceCounts - backgroundcounts
    source%rawCountRate = sourceCounts / source%exposure
    source%rawCountRateErr = ComputeSourceRateError(sourceCounts, backgroundCounts, source%exposure) 




    source%Significance = ComputeSourceSignificance(source%rawCountRate, &
                      source%rawBackGroundCountRate, source%exposure)
 


!write(*, *) "Pointsourcephotometry1 backgroundcountsperpixel=", backGroundCountsPerPixel, " apertureArea=", apertureArea
!write(*, *) "rawcountrate=", source%rawCountRate, " exposure=", source%exposure
!write(*, *) "Total counts within aperture=", sourceCounts

end subroutine PointSourcePhotometry1

!*********************************************
! Point2SourcePhotometry
! Computes the counts for a point-source
! having an aperture radius > coincidence-loss
! radius
!*********************************************
subroutine PointSourcePhotometry2(source, radius)

    ! Input (unmodfied) variables
    real(single), intent(in) :: radius 
    type(SourceParameters), intent(inout) :: source
      
    real(double) :: rawCounts, correctedCounts, countRate
    real(double) :: f
    real(double) :: backgroundRate, backgroundCounts, backgroundMode
    real(single) :: backGroundCountsError
    real(double) :: area, pixelArea
    real(double) :: correction
 
    ! Local variables
    integer (int32) :: i, j
    integer (int32) :: i1, i2, j1, j2
    real(single) :: xPos, yPos, x0, y0 !   

!write(*, *) "mssllib  PointSourcePhotometry2 line 1433 radius=", radius, " x=", source%xpos, " y=", source%ypos

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 10000000.0_double 
    maxExposure = 0.0_double
    


    xPos = source%xPos - 0.5
    yPos = source%yPos - 0.5
    apertureRadius = radius        
           
    ! Determine upper and lower array indices for
    ! summing counts
   
    i1 = int(xPos - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(xPos + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(yPos - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(yPos + radius + 0.0001)
    j2 = min(ju, j2)
 

    apertureArea = 0.0_double
    rawCounts = 0.0_double
    correctedCounts = 0.0_double
    area = 0.0_double
 
    do i = i1, i2
        x0 = real(i) + 0.5

        do j = j1, j2
            y0 = real(j) + 0.5 

           
            ! Compute the coincidence-loss correction for this pixel
            countRate = GetApertureCounts(x0, y0, coincLossRadius) / source%exposure
            apertureRadius = radius                   

!            write(*, *) "Line 1116 i=", i, " j=", j, " x0=", x0, " y0=", y0             
            pixelArea = EffectivePixelArea(x0 - xPos, y0 - yPos)

            f = pixelArea * ccdImage(i, j) 


            !*********************************************************************
            ! Maybe using an exposure map- get the mean, min and max  exposure
            !********************************************************************  
            call getExposure(i, j)
  


            
            rawCounts = rawCounts + f
            correctedCounts = correctedCounts + f * &
                      CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
                   
            area = area + pixelArea
          
        end do
    end do
!write(*, *) "radius=", radius
!write(*, *) "Line 1128 area=", area    
!write(*, *) "raw counts=", rawCounts, " corrected=", correctedCounts

    ! Get the background   
    if (source%rawBackgroundCountRate .gt. -0.0001) then ! Use the sepecified value
        backgroundMode = source%rawBackgroundCountRate

    elseif(associated(backgroundImage)) then ! Use the background image
        
        backGroundMode = GetBackGroundMode4(backgroundImage, source%xPos - 0.5, source%yPos - 0.5, &
                               source%innerBckRadius, source%outerBckRadius, backGroundCountsError)
      
    else if (associated(sourceNumimage)) then  ! Use the image
        backGroundMode = GetBackGroundMode(sourceNumImage,               &
                               source%xPos - 0.5, source%yPos - 0.5,         &
                               source%innerBckRadius, source%outerBckRadius, &
                               backGroundCountsError)

    else
        backGroundMode = GetBackGroundMode(source%xPos - 0.5, source%yPos - 0.5, &
                               source%innerBckRadius, source%outerBckRadius,         &
                               backGroundCountsError)

    end if


    backgroundCounts = backGroundMode * area
    source%rawBackGroundCountRate = backGroundCounts / source%exposure
    source%rawBackGroundCountRateErr = backGroundCountsError * area / source%exposure
  
    ! Subtract the background
    rawCounts = rawCounts - backGroundCounts 

  
    source%rawCountRateErr = ComputeSourceRateError(rawCounts, backGroundCounts, source%exposure)

  


    ! Compute the corrected rates
    backGroundRate = PI * coincLossRadius * coincLossRadius * backGroundMode / source%exposure

    ! The coincidence-loss correction for each background pixel
    correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)

    source%corrBackGroundCountRate = source%rawBackGroundCountRate * correction

    source%corrBackGroundCountRateErr = source%rawBackGroundCountRateErr * correction

    source%corrCountRate =  source%rawCountRate * correction

    source%corrCountRateErr = ComputeSourceRateError(correctedCounts, backgroundCounts, source%exposure)
           
    
    ! Compute the signal-to-noise (source counts / sqrt(source counts + background counts)   
   
    source%Significance = ComputeSourceSignificance(source%rawCountRate, &
                      source%rawBackGroundCountRate, source%exposure)
                  
    source%apertureRadius = radius * float(binX)  
    
    source%coincidenceLossCorr = (source%corrCountRate - source%rawCountRate) * 100.0 &
             / source%rawCountRate 
    source%rawCountRate = source%rawCountRate / source%exposure
    source%corrCountRate = source%corrCountRate / source%exposure

end subroutine PointSourcePhotometry2       
!*************************************************************
! Point-source photometry
!*************************************************************
subroutine PointSourcePhotometry3(source, radius, backgroundMode) 
    type(SourceParameters), intent(inout) :: source
    real(single), intent(in) ::radius, backGroundMode
    real(double) :: sourceCounts, backgroundCounts
    real(single) :: backgroundCountsError
    
    real(double) :: backGroundCountsPerPixel
    real(single) r1, r2




    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
    


    apertureRadius = radius

    if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
        r1 = source%innerBckRadius
        r2 = source%outerBckRadius
    else 
        r1 =  innerRadius
        r2 =  outerRadius
    end if
  
    ! Get the total counts within the aperture   


    sourceCounts = GetApertureCounts(source%xPos - 0.5, source%yPos - 0.5, radius)
 

    ! Get the background   

    backGroundCountsPerPixel = backgroundMode

    backGroundCounts = backGroundCountsPerPixel * apertureArea

    source%rawBackGroundCountRate = backGroundCounts / source%exposure
    backGroundCountsError = backGroundCountsError * apertureArea 
  
    ! Subtract the background and convert to count-rate
    sourceCounts = sourceCounts - backgroundcounts
    source%rawCountRate = sourceCounts / source%exposure
    source%rawCountRateErr = ComputeSourceRateError(sourceCounts, backgroundCounts, source%exposure) 


end subroutine PointSourcePhotometry3


!*****************************************
! setQualityFlag
! Set quality-flag bit 0 (bad pixel)
!
!***************************************
subroutine setQualityFlag1

    currentSource%qualityFlag = ibset(currentSource%qualityFlag, 0)
    
end subroutine setQualityFlag1


!*****************************************
! setQualityFlag2
! Set quality-flag bit 0 (bad pixel)
!
!***************************************
subroutine setQualityFlag2(source)
    type(SourceParameters), intent(inout) :: source

    source%qualityFlag = ibset(source%qualityFlag, 0)
    
end subroutine setQualityFlag2

!**********************************
!
! subroutine GetCounts
!
!********************************** 
subroutine GetCounts1(image, source)
    real(single), pointer :: image(:, :)
    type(SourceParameters), intent(inout) :: source
!    real(single) :: trueCountRate
  
    CCDImage => image

    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

    apertureRadius = source%apertureRadius
     
    source%rawBackgroundCountRate = &
             GetApertureCounts(source%xPos - 0.5, source%yPos - 0.5, apertureRadius )
               

    source%rawBackGroundCountRateErr = sqrt( source%rawBackgroundCountRate) 
 
    
    
   
    
end subroutine GetCounts1
!***************************************************
! subroutine GetCounts2
! Computes the total counts within a circle 
! of radius radius centred on xpos, ypos
!*************************************************** 
subroutine GetCounts2(image, xPos, yPos, radius, counts, area)
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: xPos, yPos, radius
    real(single), intent(out) :: counts
    real(single), optional, intent(out) :: area

    CCDImage => image


    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

    apertureRadius = radius

!write(*, *) "Getcounts2 line 2593 xpos=", xpos, " ypos=", yPos, " iu=", iu, " ju=", ju


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
  

     
    counts = GetApertureCounts(xPos, yPos, radius )
    nullify(CCDImage)           

    if (present(area)) area = apertureArea   ! The actual aperture area used

end subroutine GetCounts2

!**********************************
! subroutine GetBackgroundCounts1
!********************************** 
subroutine GetBackgroundCounts1(image, xPos, yPos, &
                                radius, radius1, radius2, counts)
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: xPos, yPos, radius, radius1
    real(single), intent(inout) :: radius2
    real(single), intent(out) :: counts
    real(single) :: error
  
    CCDImage => image

    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

    apertureRadius = radius

    maxNumPixels = 30000
    if (.not. associated(backgroundPixel)) then
        backgroundPixel => AllocateSingle(1, maxNumPixels, "backgroundPixel (AperturePhotometry)")
     
    end if


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
  


    ! Compute the background counts per pixel from the annulus    
    counts = GetBackGroundMode3(   &
              xPos,    &   ! The centre x-pixel coordinate of the aperture
              yPos,    &   ! The centre y-pixel corrdinate of the aperture 
              radius1, &   ! The aperture inner radius
              radius2, &   ! The aperture outer radius
              error)       ! The error
               

!  write(*, *) "Line 1931 Getbackgroundcounts1 mode=", counts, " radius=", radius

    counts = counts * PI * radius * radius


    deallocate(backgroundPixel)
    nullify(backgroundPixel)
    nullify(CCDImage)    

end subroutine GetBackgroundCounts1



!**********************************
! subroutine GetBackgroundCounts1
!********************************** 
subroutine GetBackgroundCounts2(image, xPos, yPos, &
                                area, radius1, radius2, counts)
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: xPos, yPos, area, radius1
    real(single), intent(inout) :: radius2
    real(single), intent(out) :: counts
    real(single) :: error
  
    CCDImage => image

    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

 

    maxNumPixels = 30000
    if (.not. associated(backgroundPixel)) then
        backgroundPixel => AllocateSingle(1, maxNumPixels, "backgroundPixel (AperturePhotometry)")
     
    end if


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
  


    ! Compute the background counts per pixel from the annulus    
    counts = GetBackGroundMode3(   &
              xPos,    &   ! The centre x-pixel coordinate of the aperture
              yPos,    &   ! The centre y-pixel corrdinate of the aperture 
              radius1, &   ! The aperture inner radius
              radius2, &   ! The aperture outer radius
              error)       ! The error
               

!  write(*, *) "Line 1931 Getbackgroundcounts2 mode=", counts, " radius=", radius

    counts = counts * area


    deallocate(backgroundPixel)
    nullify(backgroundPixel)
    nullify(CCDImage)    

end subroutine GetBackgroundCounts2





!******************************************************
!
!
!
!
!*****************************************************

subroutine AperturePhotometry3(source, n, image, levelImage, &
                               sourceQualityImage, &
                               radius, minSignificance)

    type(SourceParameters), pointer :: source(:)
    integer, intent(in) :: n       
    real(single), pointer :: image(:, :) 
    integer(int16), pointer :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
 
    real(single), intent(in) :: radius, minSignificance
    real(single) :: sigma = 0.0
    integer :: i

    if( associated(exposureImage) ) then
        source%exposure = 1.0
    end if

    qualityImage => sourceQualityImage ! The quality image (false=good pixels, true=bad pixels)
 
    call Initialise(int(size(source), int16), int(size(image, 1), int16), int(size(image, 2), int16))
    CCDImage => image 

    sourceNumImage => levelImage


    call refineSourceNumImage(source, int(n, int16), sourceNumimage)



    ! Compute the mode of the image, ignoring source regions

    backgroundCountsMode = ComputeBackgroundMode(image, &
                     sourceNumImage, nx, ny, backgroundSigma, sigma)


!write(*, *) "Line 2978 computed global background=", backgroundCountsMode
   
!write(*, *) "**********************************************************************"
!write(*, *) "aperturephotometry3"
!write(*, *) "**********************************************************************"

 
    do i = 1, n

!if(abs(source(i)%xpos - 387.0) > 5.0 .and. abs(source(i)%ypos - 274.0) > 5.0) cycle        
        if (source(i)%extensionFlag .gt. 0) cycle

!        if ( btest(source(i)%confusionFlag, 2) ) cycle


!write(*, *) "Source ", i, " number=", source(i)%number, &
!     " x=", source(i)%xpos, " y=", source(i)%ypos
!write(*, *) "old corrrate=", source(i)%corrcountrate, " background=", source(i)%corrbackgroundcountrate, &
!  " old significance=", source(i)%significance, " aperture radius=", source(i)%apertureRadius
   

        if (.not. isNullValue(source(i)%significance) ) then
            if (source(i)%significance .gt. minSignificance) cycle
            if( source(i)%apertureRadius / real(binx) <= radius) cycle
        end if






        currentSource => source(i) 

      
        currentSource%innerBckRadius = 0.0
        currentSource%outerBckRadius = 0.0
      
     

  

!write(*, *) "Recomputing photometry on source ", i, " old raw count-rate=", source(i)%rawCountRate, &
! " old raw background count-rate=", source(i)%rawBackgroundCountRate

        currentSource%rawBackgroundCountRate = -1.0


        if (associated(exposureImage) ) then    
            call PointSourcePhotometry4(currentsource, radius)
        else 

            call PointSourcePhotometry1(currentsource, radius)

        end if
        currentsource%ApertureRadius = radius * real(binX)

        !***********************************************************************
        !
        ! If using an exposure image then multiply significance by sqrt(mean exposure)
        !
        !*************************************************************************
!        if (associated(exposureImage) ) then
!            call scaleErrors( currentSource )        
!        end if





        call PointSourceCoincLossCorr( currentsource, 12.0, frameTime)     

        nullify(currentSource)

       

!write(*, *) "Source ", i, " x=", source(i)%xpos, " y=", source(i)%ypos
!write(*, *) "new corrrate=", source(i)%corrcountrate, " background=", source(i)%corrbackgroundcountrate, &
!  " new significance=", source(i)%significance
   

  

!write(*, *) "newsignificance=", source(i)%significance

             
!write(*, *) "New count-rate=", source(i)%rawCountRate   


    end do

    nullify(CCDImage, qualityImage, sourceNumImage, currentSource)
    call deleteArrays


end subroutine AperturePhotometry3




!**************************************************************
!
! Fow when using the background image for aperture photometry
!
!
!**************************************************************

subroutine AperturePhotometry4(source, n, image, &
                               backgroundImageIn, &
                               levelImage, &
                               sourceQualityImage, &
                               radius, minSignificance)

    type(SourceParameters), pointer :: source(:)
    integer, intent(in) :: n       
    real(single), pointer, dimension(:, :) :: image, backGroundImageIn 
    integer(int16), pointer :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
 
    real(single), intent(in) :: radius, minSignificance

    backGroundImage => backgroundImageIn 

  

    call AperturePhotometry3(source, n, image, levelImage, &
                               sourceQualityImage, &
                               radius, minSignificance)


    nullify(backgroundImageIn)

end subroutine AperturePhotometry4
!**********************************************************************
!
!
!
!
!*********************************************************************
subroutine refineSourceNumImage(source, nsources, levelimage)
    type(SourceParameters), pointer :: source(:)
    integer(int16), intent(in) :: nSources       
    integer(int16), pointer :: levelImage(:, :) 
    real(single) :: apertureRadius, radius, dx, dy

    integer(int16) :: i, j, k, i0, j0, il, iu, jl, ju, nx, ny, dx1


    
    nx = ubound(levelImage, 1)
    ny = ubound(levelImage, 2)

!if(nx < 50) return
    apertureRadius = real(12 / binx)
    dx1 = 12 / binx + 1 

!write(*, *) "refinesourcenumimage"
    where(levelimage > 0) levelImage = 0

    do i = 1, nSources
!write(*, *) i, ") number=", source(i)%number, " x=", source(i)%xpos, " y=", source(i)%ypos
        if (source(i)%number < 0 .or. source(i)%extensionFlag == 1) cycle
        i0 = int(source(i)%xpos - 0.5)
        j0 = int(source(i)%ypos - 0.5)
        il = max(0_int16, i0 - dx1)
        iu = min(nx, i0 + dx1)
        jl = max(0_int16, j0 - dx1)
        ju = min(ny, j0 + dx1)


        do j = il, iu
            dx = source(i)%xpos - real(j + 1)
            if ( abs(dx) > apertureRadius) cycle
            do k = jl, ju
                if (levelImage(j, k) .ne. 0) cycle
                dy = source(i)%ypos - real(k + 1)
                if ( abs(dy) > apertureRadius) cycle
                radius = sqrt(dx * dx + dy * dy)
                if ( radius <= apertureRadius) then
                    levelImage(j, k) = source(i)%number
                end if 
         
             end do
        end do
 
    end do







!nx = nx + 1
!ny = ny + 1
!call Saveint16Image(levelImage, nx, ny, "levelimage.fits", "test")
!call fatal("stopped at line 3162")
end subroutine refineSourceNumImage


!**********************************************************
!
! Routine when exposure-image is being used to
! get the mean exposure within the aperture area
!
!
!**********************************************************

subroutine getExposure(i, j)
    integer, intent(in) :: i, j


    if ( .not. associated(exposureImage) ) return

    if( isNullValue( exposureImage(i, j) )) return

    numExposurePixels = numExposurePixels + 1
    meanExposure = meanExposure + dble(exposureImage(i, j ))
    minExposure = min( minExposure, dble(exposureImage(i, j ) ) )
    maxExposure = max( maxExposure, dble(exposureImage(i, j ) ) )

end subroutine getExposure
!***********************************************************************
! This routine has been added in July 2008 to scale the count-rate errors
! when an exposure image is being used. 
! In such cases the exposure time of
! each source will have been set to 1.
! The variable exposureConstant will have been set to the mean exposure of
! the exposure image.
! The variable meanExposure will equal the mean exposure within the
! aperture area
! NB source%exposure will have been set equal to 1.0
!
!
!***********************************************************************
subroutine scaleErrors( source )
    type(SourceParameters), intent(inout) :: source

    real(double) :: constant, constant1
 !   real(single) :: oldSignificance

!    write(*, *) "omphotometry - Scaling rate errors, etc, extensionflag=", source%extensionflag
!    write(*, *) "x=", source%xpos, " y=", source%ypos
!    write(*, *) "Input raw rate=", source%rawCountrate, " significance=", source%significance, &
!        " background=", source%rawBackgroundCountRate, " exposure=", source%exposure

    if (numExposurePixels .gt. 0) then

        

        !************************************************************************
        !
        ! Compute the mean exposure time within the photometry aperture
        !
        !***********************************************************************
        meanExposure = meanExposure / dble(numExposurePixels)



!        write(*, *) "mean exposure=", meanExposure, " numexposurepixels=", numExposurePixels
!        write(*, *) "exposureconstant=", exposureconstant       
     
!oldSignificance = source%significance



        !**********************************************************************
        !
        ! Need to divide all rates by the exposure constant
        !
        ! Rate * (mean exposure / exposure constant) gives the total counts
        ! (since the source%exposure is set to 1.0) and then divide by the
        ! mean exposure to get the rate 
        ! equals rate / exposure constant 
        !
        !**********************************************************************
 
 

        source%rawCountRate = source%rawCountRate / exposureConstant 
        source%rawBackgroundCountRate = source%rawBackgroundCountRate / exposureConstant 

        !******************************************************************
        !
        ! Scale the significance
        !
        !******************************************************************
        constant = meanExposure * meanExposure / exposureConstant   
        constant1 = sqrt( constant )  

!write(*, *) "constant=", constant, " constant1=", constant1
!write(*, *) "initial rawcountrateerr=", source%rawCountRateErr, " source%exposure=", source%exposure

        source%significance = source%significance * constant / constant1  

        !******************************************************************
        !
        ! Scale the count-rate errors
        !
        !******************************************************************
        source%rawCountRateErr = source%rawCountRateErr / exposureConstant
        source%rawBackgroundCountRateErr = source%rawBackgroundCountRateErr / exposureConstant



!write(*, *) "Finalrawcountrateerr=", source%rawCountRateErr




        !***************************************************************************
        !
        ! Set the exposure-time equal to the mean exposure-time within the aperture
        !
        !***************************************************************************
        source%exposure = meanExposure
         

!        write(*, *) "Line 504 extension flag=" , Source%extensionFlag, &
!                  "   rawrate=", source%rawcountrate,      " significance=", source%significance
!        write(*, *) "numexposurepixels=", numExposurePixels, " meanexposure=", meanExposure 



!write(*, *) "OLD SIG=", oldSignificance, " new=", source%significance


    end if


end subroutine scaleErrors


!*************************************************************
!
! Point-source aperture photometry (USING AN EXPOSURE IMAGE)
! 
! Input parameter
!        radius - aperture radius
! Input/output parameters
!        source - structure containing source-parameters (coordinates, etc)
!
! Notes
! Uses the EXPOSURE image
!
! The input image will originally have been in counts/second
! Omdetect will have multiplied it by the median exposure
!
! Method
! 1) Find the background counts per pixel, using the pixels within an annulus
! 
! 2) Sum the rate within the aperture - ie 
! 3) For each pixel-
!                 Subtract the background counts per pixel from the background
!                 Multiply this value by the exposure pixel value / divided by the mean exposure
!
!   
!*************************************************************
subroutine PointSourcePhotometry4(source, radius) 

    type(SourceParameters), intent(inout) :: source
    real(single), intent(in) ::radius
  
    
    real(single) :: backGroundCountsPerPixel,  backGroundCountsError 
    real(single) r1, r2, area



    apertureRadius = radius

    area = PI * radius * radius ! The required aperture area


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 100000000.0_double 
    maxExposure = 0.0_double
    

    if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
        r1 = source%innerBckRadius
        r2 = source%outerBckRadius
    else 
        r1 =  innerRadius
        r2 =  outerRadius
    end if

  

    !***********************************************************
    !
    ! Get the background
    !   
    !***********************************************************
    if (source%rawBackgroundCountRate .gt. -0.00001) then
        backgroundCountsPerPixel = source%rawBackgroundCountRate


    else if(associated(backgroundImage)) then ! Use the background image
        
        backGroundCountsPerPixel = GetBackGroundMode4(backgroundImage, source%xPos - 0.5, source%yPos - 0.5, &
                              r1, r2, backGroundCountsError)
    else if (associated(sourceNumImage)) then

        backGroundCountsPerPixel = GetBackGroundMode1(sourcenumImage, source%xPos - 0.5, source%yPos - 0.5, &
                              r1, r2, backGroundCountsError)
    else   
        backGroundCountsPerPixel = GetBackGroundMode3(source%xPos - 0.5, source%yPos - 0.5, &
                              r1, r2, backGroundCountsError)

    end if







    !***********************************************
    !
    ! backgroundCountsPerPixel is for an exposure of exposureConstant
    ! Get the background-subtracted raw count-rate and  
    ! the also background raw-rate
    !**********************************************
    call GetApertureCountrate(                         &
                    radius,                            &
                    backgroundCountsPerPixel,          &  
                    backGroundCountsError,             &
                    source)
 


    source%exposure = meanExposure 




!    write(*,  *) "PointSourcePhotometry4 line 3399"
!    write(*, *) "x=", source%xpos, " y=", source%ypos
!    write(*, *) "rawrate=", source%rawCountRate, "+/-", source%rawCountRateErr,           &
!                " background=", source%rawbackgroundCountRate,  "+/-", source%rawBackgroundCountRateErr, &
!                " meanexposure=", meanExposure, " backgroundcountsperpixel=",               &
!                  backgroundCountsPerPixel
!   write(*, *) "aperturearea=", apertureArea, " required aperture radius=", radius





!if( backgroundCountsPerPixel < 0.0 .or. backgroundCountsPerPixel > 10000.0 .or. &
!       isNullValue(backgroundCountsPerPixel)  ) then
!write(*, *) "r1=", r1, " r2=", r2
!call fatal("bug")
!end if




end subroutine PointSourcePhotometry4


!*********************************************
!
! subroutine GetApertureCountRate
! For use with an exposure image and for point sources
! Computes the counts within a given aperture
! 
!
!*********************************************
subroutine      GetApertureCountRate(  &
                radius,                        & ! The aperture radius (pixels) for summing the counts
                backgroundCountsPerPixel,      & ! The background counts per pixel from the annulus
                backgroundCountsPerPixelError, & ! The background counts per pixel error from the annulus
                source)             


    !********************************************* 
    !
    ! Input (unmodfied) variables
    !*********************************************
    real(single), intent(in) :: radius
    real(single), intent(in) :: backgroundCountsPerPixel, backgroundCountsPerPixelError 
   
    !********************************************* 
    !
    ! Input/output (unmodfied) variables
    !*********************************************
    type(SourceParameters), intent(inout) :: source


    !****************************************
    ! Local variables
    !****************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
    real(double) :: imageCounts, backgroundCounts, backgroundSubtractedCounts
    real(double) :: constant, area   
    real(double) :: sum1, sum2, sum3, sum4, totalArea
    real(double) :: sourceRate, backgroundRate 
    real(double) :: sourceRateError, rateError
    real(single) :: x0, y0




!ccdImage = 10.0
!exposureImage = 1000.0
!exposureConstant = 1000.0 
    apertureRadius = radius  ! For internal use      
           
    ! Determine upper and lower array indices for
    ! summing counts
   
    x0 = source%xpos - 0.5
    y0 = source%ypos - 0.5


    i1 = int(x0 - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(x0 + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(y0 - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(y0 + radius + 0.0001)
    j2 = min(ju, j2)




    backgroundRate = 0.0_double
    sourceRate = 0.0_double ! Initialise to zero 

    apertureArea = 0.0_double

    !**************************************************
    !
    ! Sum the pixels in the aperture round the centre
    !
    !**************************************************

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   
    sum1 = 0.0_double ! For storing the sum of the error squared of the background-subtracted raw count-rate
    sum2 = 0.0_double ! For storing the sum of the error squared of the background raw count-rate

    sum3 = 0.0_double ! For storing the total image counts within the aperture
    sum4 = 0.0_double ! For storing the total background-subtracted counts within the aperture


    totalArea = 0.0_double
    do i = i1, i2
        do j = j1, j2         


            !**********************************************************
            !
            ! Check the exposure pixel
            !
            !**********************************************************
            if ( isNullValue( exposureImage(i, j) ) .or. &
                 isNullvalue(ccdImage(i, j) ) ) then
                cycle
            end if

            !************************************************************************
            !
            ! The following corrects for the true exposure
            !
            !************************************************************************
            constant = dble(exposureImage(i, j) ) /  exposureConstant 


            !*************************************************************
            !
            ! Get the pixel value (multiplied by the area of the pixel- 
            ! 1 if totally inside aperture 0 if totally outside aperture)
            ! Remember that the original image had been divided by the total exposure
            ! and omdetect has multiplied it by exposureConstant
            ! area is returned and is the actual pixel area (0-1)
            !*************************************************************
            imageCounts = PixelValue1(x0, y0, i, j, area) * constant ! The pixel counts -
            if(area .lt. 1e-6) cycle

            totalArea = totalArea + area 
            sum3 = sum3 + imageCounts 

            !***************************************************  
            !              
            ! Compute the background counts
            ! 
            !***************************************************
            backgroundCounts = backgroundCountsPerPixel * area * constant
                

            !***************************************************  
            !              
            ! Compute the background subtrated counts 
            ! 
            !***************************************************
            backgroundSubtractedCounts = max(0.0_double, imageCounts - backgroundCounts)
!            backgroundSubtractedCounts = imageCounts - backgroundCounts

            sum4 = sum4 + backgroundSubtractedCounts
    

            !************************************************************************
            !
            ! Convert background-subtracted  counts to count-rate and add to the sum 
            !
            !************************************************************************
            sourceRate = sourceRate + backgroundSubtractedCounts / &
                           dble(exposureImage(i, j) )


            !************************************************************************
            !
            ! Convert background counts to count-rate and add to the sum 
            !
            !************************************************************************
            backgroundRate = backgroundRate + backgroundcounts / &
                                    dble(exposureImage(i, j) )
 

            

            !************************************************************************
            !
            ! Compute the background-subtracted count-rate error and add the square
            ! of it to sum1
            !************************************************************************
            sourceRateError = computeSourceRateError( backgroundSubtractedCounts, &
                                   backgroundCounts, exposureImage(i, j) ) 
                
            sum1 = sum1 + sourceRateError * sourceRateError 

            !***************************************************************************
            !
            ! Compute the background count-rate error and add the square of it to sum2 
            !
            !***************************************************************************
            rateError = backgroundCountsPerPixelError * area * constant / dble(exposureImage(i, j)) 
            sum2 = sum2 + rateError * rateError



            numExposurePixels = numExposurePixels + 1

            meanExposure = meanExposure + dble(exposureImage(i, j))


            minExposure = min(minExposure, dble(exposureImage(i, j)) ) 
            maxExposure = max(maxExposure, dble(exposureImage(i, j)) ) 


            !*******************************************************
            ! Check the qualaity pixel
            !******************************************************
            if (associated(qualityImage) ) then    
                if (qualityImage(i, j)) then ! Bad pixel- set source quality flag
                call setQualityFlag(source)    
            end if
        end if





     
        end do

    end do

    source%rawCountRate = sourceRate
    source%CorrCountRate = sourceRate


    source%rawbackgroundCountRate = backgroundRate
    source%corrbackgroundCountRate = backgroundRate



    !*********************************************************
    !
    ! The errors in the raw rates
    !
    !**********************************************************
    source%rawCountRateErr = sqrt(sum1)
    source%CorrCountRateErr = source%rawCountRateErr


    source%rawBackgroundCountRateErr = sqrt(sum2)
    source%CorrBackgroundCountRateErr = source%rawBackgroundCountRateErr


    !****************************************************************
    !
    ! Compute the significance (signal-to-noise)
    !
    !******************************************************************
    if(sum3 > 0.0) then
        source%significance = sum4 / sqrt(sum3)
    else
        source%significance = 0.0
    end if

    if(sourceRate > 0.0) then
        source%exposure = sum4 / sourceRate
    end if

    if( (maxExposure - minExposure) > 10.0) then
        source%qualityFlag = ibset( source%qualityFlag, 10)
    end if


!write(*, *) "Getaperturecountrate line 3667 minexposure=", minExposure, " max=", maxExposure, &
!" numexposurepixels=", numexposurePixels, " totalArea=", totalArea, " aperture radius=", radius

    apertureArea = totalArea

end subroutine GetApertureCountRate     







!*************************************************************
!
! Extended-source photometry (USING AN EXPOSURE IMAGE)
! 
! Input parameter
!        radius - aperture radius
! Input/output parameters
!        source - structure containing source-parameters (coordinates, etc)
!
! Notes
! Uses the EXPOSURE image
!
! The input image will originally have been in counts/second
! Omdetect will have multiplied it by the median exposure
!
! Method
! 1) Find the background counts per pixel, using the pixels within an annulus
! 
! 2) Sum the rate within the aperture - ie 
! 3) For each pixel-
!                 Subtract the background counts per pixel from the background
!                 Multiply this value by the exposure pixel value / divided by the mean exposure
!
!   
!*************************************************************
subroutine ExtendedSourcePhotometry4(source) 

    type(SourceParameters), intent(inout) :: source
   
  
    real(single) :: backGroundCountsPerPixel,  backGroundCountsError 
    real(single) r1, r2, area, theta


    area = PI * source%semiA * source%semiB ! The required aperture area

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 100000000.0_double 
    maxExposure = 0.0_double
    


   
  
!write(*, *) "Extendedsourcephotometry4, r1=", r1, " r2=", r2
!write(*, *) "x=", source%xpos, " y=", source%ypos, &
!" a=", source%semia, " b=", source%semib, " theta=", source%theta, " number=", source%number
!write(*, *) "Innerbckradius=", source%innerBckRadius, " outset=", source%outerBckRadius



    !***********************************************
    !
    ! backgroundCountsPerPixel is for an exposure of exposureConstant
    ! Get the background-subtracted raw count-rate and  
    ! the also background raw-rate
    !**********************************************

    if( associated(backgroundImage) ) then
         call GetExtendedCountrate2( source )

    else

        if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
            r1 = source%innerBckRadius
            r2 = source%outerBckRadius
        else 
            r1 =  source%semib  + real(4 / binx)
            r2 =  source%semib  + real(10 / binx)
        end if


        theta = source%theta
        if(theta < 0.0) theta = theta + 180.0

 
        source%axialRatio = source%semiA / source%semiB
        backgroundCountsPerPixel = GetBackGroundMode2( &
                     source%xPos,                    &   ! The centre x-pixel coordinate of the aperture
                     source%yPos,                    &   ! The centre y-pixel corrdinate of the aperture 
                     r1,                             &   ! The aperture inner radius
                     r2,                             &   ! The aperture outer radius
                     source%axialRatio,              &   ! The ellipse axial ratio
                     theta,                          &   ! The ellipse position angle
                     backgroundCountsError)      ! The error

         call GetExtendedCountrate1(                   &
                    backgroundCountsPerPixel,          &  
                    backGroundCountsError,             &
                    source)
 
    end if

    source%exposure = meanExposure 


!    write(*, *) "x=", source%xpos, " y=", source%ypos
!    write(*, *) "rawrate=", source%rawCountRate, "+/-", source%rawCountRateErr,           &
!                " background=", source%rawbackgroundCountRate,  "+/-", source%rawBackgroundCountRateErr, &
!                " meanexposure=", meanExposure, " backgroundcountsperpixel=",               &
!                  backgroundCountsPerPixel
!   write(*, *) "aperturearea=", apertureArea, " extensionflag=", source%extensionFlag

end subroutine ExtendedSourcePhotometry4

!*********************************************
!
! subroutine GetExtendedCountRate
! For use with an exposure image
! Computes the counts within a given aperture
! 
!
!*********************************************
subroutine      GetExtendedCountRate1(         &
                backgroundCountsPerPixel,      & ! The background counts per pixel from the annulus
                backgroundCountsPerPixelError, & ! The background counts per pixel error from the annulus
                source)             


    !********************************************* 
    !
    ! Input (unmodfied) variables
    !********************************************
    real(single), intent(in) :: backgroundCountsPerPixel, backgroundCountsPerPixelError 
   
    !********************************************* 
    !
    ! Input/output (unmodfied) variables
    !*********************************************
    type(SourceParameters), intent(inout) :: source


    !****************************************
    ! Local variables
    !****************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
    real(double) :: imageCounts, backgroundCounts, backgroundSubtractedCounts
    real(double) :: constant
    real(double) :: sum1, sum2, sum3, sum4
    real(double) :: sourceRate, backgroundRate 
    real(double) :: sourceRateError, rateError
    real(single) :: x0, y0




           
    ! Determine upper and lower array indices for
    ! summing counts
   
    x0 = source%xpos - 0.5
    y0 = source%ypos - 0.5


    i1 = source%imin
    i1 = max(il, i1)
  
    i2 = source%imax
    i2 = min(iu, i2)  

    j1 = source%jmin
    j1 = max(jl, j1)

    j2 = source%jMax
    j2 = min(ju, j2)




    backgroundRate = 0.0_double
    sourceRate = 0.0_double ! Initialise to zero 

    apertureArea = 0.0_double

    !**************************************************
    !
    ! Sum the pixels in the strip round the centre
    !
    !**************************************************

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   
    sum1 = 0.0_double ! For storing the sum of the error squared of the background-subtracted raw count-rate
    sum2 = 0.0_double ! For storing the sum of the error squared of the background raw count-rate

    sum3 = 0.0_double ! For storing the total image counts within the aperture
    sum4 = 0.0_double ! For storing the total background-subtracted counts within the aperture


    do i = i1, i2
        do j = j1, j2
                   
            if (sourceNumImage(i, j) .ne. source%number) then ! The pixel is not part of the sources
                cycle
            end if
            !**********************************************************
            !
            ! Check the exposure pixel
            !
            !**********************************************************
            if ( isNullValue( exposureImage(i, j) ) .or. &
                 isNullvalue( ccdImage(i, j) ) ) then
                cycle
            end if

            !************************************************************************
            !
            ! The following corrects for the true exposure
            !
            !************************************************************************
            constant = dble(exposureImage(i, j) ) /  exposureConstant 


            !*************************************************************
            !
            ! Get the pixel value 
            !
            !*************************************************************
            imageCounts = dble(ccdImage(i, j)) * constant ! The pixel counts -
             


            sum3 = sum3 + imageCounts 

            !***************************************************  
            !              
            ! Compute the background counts
            ! 
            !***************************************************
            backgroundCounts = backgroundCountsPerPixel * constant
                

            !***************************************************  
            !              
            ! Compute the background subtrated counts 
            ! 
            !***************************************************
            backgroundSubtractedCounts = max(0.0_double, imageCounts - backgroundCounts)

!            backgroundSubtractedCounts = imageCounts - backgroundCounts


            sum4 = sum4 +  backgroundSubtractedCounts
    
            !************************************************************************
            !
            ! Convert background-subtracted  counts to count-rate and add to the sum 
            !
            !************************************************************************
            sourceRate = sourceRate + backgroundSubtractedCounts / &
                           dble(exposureImage(i, j) )


            !************************************************************************
            !
            ! Convert background counts to count-rate and add to the sum 
            !
            !************************************************************************
            backgroundRate = backgroundRate + backgroundcounts / &
                                dble(exposureImage(i, j) )

            !************************************************************************
            !
            ! Compute the background-subtracted count-rate error and add the square
            ! of it to sum1
            !************************************************************************
            sourceRateError = computeSourceRateError( backgroundSubtractedCounts, &
                               backgroundCounts, exposureImage(i, j) ) 
                
            sum1 = sum1 + sourceRateError * sourceRateError 

            !***************************************************************************
            !
            ! Compute the background count-rate error and add the square of it to sum2 
            !
            !***************************************************************************
            rateError = backgroundCountsPerPixelError * constant / dble(exposureImage(i, j)) 
            sum2 = sum2 + rateError * rateError

            numExposurePixels = numExposurePixels + 1

     

            minExposure = min(minExposure, dble(exposureImage(i, j)) ) 
            maxExposure = max(maxExposure, dble(exposureImage(i, j)) ) 


            !*******************************************************
            !
            ! Check the quality pixel
            !
            !******************************************************
            if (associated(qualityImage) ) then    
                if (qualityImage(i, j)) then ! Bad pixel- set source quality flag
                    call setQualityFlag(source)    
                end if
            end if





        
        end do

    end do

    source%rawCountRate = sourceRate
    source%CorrCountRate = sourceRate


    source%RawbackgroundCountRate = backgroundRate
    source%CorrbackgroundCountRate = backgroundRate

    !*********************************************************
    !
    ! The errors in the raw rates
    !
    !**********************************************************
    source%rawCountRateErr = sqrt(sum1)
    source%CorrCountRateErr = source%rawCountRateErr 


    source%rawBackgroundCountRateErr = sqrt(sum2)

    source%corrBackgroundCountRateErr =  source%rawBackgroundCountRateErr


    !****************************************************************
    !
    ! Compute the significance (signal-to-noise)
    !
    !******************************************************************
    if(sum3 > 0.0) then
        source%significance = sum4 / sqrt(sum3)
    else
        source%significance = 0.0
    end if

 

    if(sourceRate > 0.0) then
        source%exposure = sum4 / sourceRate
    end if
    if( (maxExposure - minExposure) > 10.0) then    
        source%qualityFlag = ibset( source%qualityFlag, 10)
    end if

end subroutine GetExtendedCountRate1     





!*********************************************
!
! subroutine GetExtendedCountRate
! For use with an exposure image
! Computes the counts within a given aperture
! 
! For use when a background image is available
!*********************************************
subroutine  GetExtendedCountRate2( source )             

   
    !********************************************* 
    !
    ! Input/output (unmodfied) variables
    !*********************************************
    type(SourceParameters), intent(inout) :: source


    !****************************************
    ! Local variables
    !****************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
    real(double) :: imageCounts, backgroundCounts, backgroundSubtractedCounts
    real(double) :: constant
    real(double) :: sum1, sum2, sum3, sum4
    real(double) :: sourceRate, backgroundRate 
    real(double) :: sourceRateError, rateError
    real(single) :: x0, y0




           
    ! Determine upper and lower array indices for
    ! summing counts
   
    x0 = source%xpos - 0.5
    y0 = source%ypos - 0.5


    i1 = source%imin
    i1 = max(il, i1)
  
    i2 = source%imax
    i2 = min(iu, i2)  

    j1 = source%jmin
    j1 = max(jl, j1)

    j2 = source%jMax
    j2 = min(ju, j2)




    backgroundRate = 0.0_double
    sourceRate = 0.0_double ! Initialise to zero 

    apertureArea = 0.0_double

    !**************************************************
    !
    ! Sum the pixels in the strip round the centre
    !
    !**************************************************

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   
    sum1 = 0.0_double ! For storing the sum of the error squared of the background-subtracted raw count-rate
    sum2 = 0.0_double ! For storing the sum of the error squared of the background raw count-rate

    sum3 = 0.0_double ! For storing the total image counts within the aperture
    sum4 = 0.0_double ! For storing the total background-subtracted counts within the aperture


    do i = i1, i2
        do j = j1, j2
                   
            if (sourceNumImage(i, j) .ne. source%number) then ! The pixel is not part of the sources
                cycle
            end if
            !**********************************************************
            !
            ! Check the exposure pixel
            !
            !**********************************************************
            if ( isNullValue( exposureImage(i, j) ) .or. &
                 isNullvalue( ccdImage(i, j) ) ) then
                cycle
            end if

            !************************************************************************
            !
            ! The following corrects for the true exposure
            !
            !************************************************************************
            constant = dble(exposureImage(i, j) ) /  exposureConstant 


            !*************************************************************
            !
            ! Get the pixel value 
            !
            !*************************************************************
            imageCounts = dble(ccdImage(i, j)) * constant ! The pixel counts -
             


            sum3 = sum3 + imageCounts 

            !***************************************************  
            !              
            ! Compute the background counts
            ! 
            !***************************************************
            backgroundCounts = dble(backgroundImage(i, j)) * constant
                

            !***************************************************  
            !              
            ! Compute the background subtrated counts 
            ! 
            !***************************************************
            backgroundSubtractedCounts = max(0.0_double, imageCounts - backgroundCounts)




            sum4 = sum4 + backgroundSubtractedCounts
    
            !************************************************************************
            !
            ! Convert background-subtracted  counts to count-rate and add to the sum 
            !
            !************************************************************************
            sourceRate = sourceRate + backgroundSubtractedCounts / &
                           dble(exposureImage(i, j) )


            !************************************************************************
            !
            ! Convert background counts to count-rate and add to the sum 
            !
            !************************************************************************
            backgroundRate = backgroundRate + backgroundcounts / &
                                dble(exposureImage(i, j) )

            !************************************************************************
            !
            ! Compute the background-subtracted count-rate error and add the square
            ! of it to sum1
            !************************************************************************
            sourceRateError = computeSourceRateError( backgroundSubtractedCounts, &
                               backgroundCounts, exposureImage(i, j) ) 
                
            sum1 = sum1 + sourceRateError * sourceRateError 

            !***************************************************************************
            !
            ! Compute the background count-rate error and add the square of it to sum2 
            !
            !***************************************************************************
            rateError = backgroundCounts / dble(exposureImage(i, j)) 
            sum2 = sum2 + rateError * rateError

            numExposurePixels = numExposurePixels + 1

     

            minExposure = min(minExposure, dble(exposureImage(i, j)) ) 
            maxExposure = max(maxExposure, dble(exposureImage(i, j)) ) 


            !*******************************************************
            !
            ! Check the quality pixel
            !
            !******************************************************
            if (associated(qualityImage) ) then    
                if (qualityImage(i, j)) then ! Bad pixel- set source quality flag
                    call setQualityFlag( source )    
                end if
            end if

        
        end do

    end do

    source%rawCountRate = sourceRate
    source%CorrCountRate = sourceRate


    source%RawbackgroundCountRate = backgroundRate
    source%CorrbackgroundCountRate = backgroundRate

    !*********************************************************
    !
    ! The errors in the raw rates
    !
    !**********************************************************
    source%rawCountRateErr = sqrt(sum1)
    source%CorrCountRateErr = source%rawCountRateErr 


    source%rawBackgroundCountRateErr = sqrt(sum2)

    source%corrBackgroundCountRateErr =  source%rawBackgroundCountRateErr


    !****************************************************************
    !
    ! Compute the significance (signal-to-noise)
    !
    !******************************************************************
    if(sum3 > 0.0) then
        source%significance = sum4 / sqrt(sum3)
    else
        source%significance = 0.0
    end if

 

    if(sourceRate > 0.0) then
        source%exposure = sum4 / sourceRate
    end if
    if( (maxExposure - minExposure) > 10.0) then    
        source%qualityFlag = ibset( source%qualityFlag, 10)
    end if

end subroutine GetExtendedCountRate2     

end module omPhotometry_mod

























