
!********************************************
! module omdetectsource_mod
! 
!
!********************************************


module omsourceparam_mod
    use dal
    use cal   
!    use OmAllocation_mod      ! For memory allocation


    interface ComputeSourceSignificance
        module  procedure ComputeSourceSignificance1,  ComputeSourceSignificance2
    end interface

    interface sourcedataarray
        module procedure SourceDataArray1, sourcedataarray2
    end interface sourcedataarray
  
  
    real (single), parameter :: nullValue = -99999.0
 
    !**********************************************************************
    ! The following structure stores the parameters for a detected source
    !**********************************************************************

    type SourceParameters
        integer(int32) :: number   = 0        ! The source number
        real(single)   :: xPos           = -1.0     ! x source pos (pixels)
        real(single)   :: yPos           = -1.0     ! y source pos (pixels)
        real(double)   :: rightAscension = nullValue     ! Right Ascension
        real(double)   :: declination    = nullValue     ! declination
        real(single)   :: poserr    = -1.0          ! error on position

        real(single)   :: exposure                          ! Exposure time (seconds)
        real(single)   :: significance      = nullValue     ! source significance
        real(single)   :: rawCountRate      = nullValue     ! raw count rate
        real(single)   :: rawCountRateErr   = nullValue     ! error on raw count rate
        real(single)   :: corrCountRate = nullValue     ! raw counts corrected for coincidence losses
        real(single)   :: corrCountRateErr = nullValue  ! error on source counts corrected for coincidence losses
        real(single)   :: magnitude                     ! Instrumental (Vega) magnitude
        real(single)   :: magnitudeErr                  ! Instrumental (Vega) magnitude error 
        real(single)   :: rawBackGroundCountRate  = nullValue! Source raw background counts
        real(single)   :: rawBackGroundCountRateErr  = nullValue ! Source raw background counts
        real(single)   :: corrBackgroundCountRate = nullValue ! corrected background counts
        real(single)   :: corrBackgroundCountRateErr = nullValue ! corrected background counts
        real(single)   :: psf1Correction = 0.0      ! psf correction to cal aperture
        real(single)   :: psf2Correction = 0.0      ! psf correction to cal aperture
        real(single)   :: coincidenceLossCorr = 0.0 ! Coincidence-loss correction 
        real(single)   :: tdsCorr = nullValue       ! Time-dependent-sensitivity correction 

        real(single)   :: semiA = nullValue         ! width of psf
        real(single)   :: semiAErr = nullValue      ! Error in width of psf

        real(single)   :: semiB = nullValue         ! width of psf
        real(single)   :: semiBErr = nullValue      ! Error in width of psf

        real(single)   :: theta  = nullValue        ! The position-angle for each source 
        real(single)   :: thetaErr  = nullValue     ! The position-angle for each source 

        real(single)   :: apertureRadius = -1.0     ! The semi-minor axis radius for each source
        real(single)   :: axialRatio = 1.0          ! The axial-ratio of aperture ellipse (1 = circle)
        real(single)   :: alpha = 0.0               ! The position-angle of the ellipse
        real(single)   :: innerBckRadius = -1.0     ! The semi-minor aperture radii for each source
        real(single)   :: outerBckRadius = -1.0     ! The semi-minor aperture radii for each source
        integer(int32) :: noSourcePixels = 0        ! The number of pixels included by an extended source
        integer(int8)  :: extensionFlag   = 0       ! Point-source (0), otherwise extended (1)
        integer(int8)  :: confusionFlag  = 0        !
        integer(int16)  :: qualityFlag  = 0         ! Set to 1 if 1 or more source pixels is bad


        integer(int16) :: iMin = 0                  ! x - lower index image array bound of source
        integer(int16) :: iMax = 0                  ! x - upper index image array bound of source
        integer(int16) :: jMin = 0                  ! y - upper index image array bound of source
        integer(int16) :: jMax = 0                  ! y - upper index image array bound of source


    end type SourceParameters
    
    ! Linked list structure for storing the sources
    type Source

        type (Source), pointer :: next => Null()
        type(sourceParameters) :: data
    end type Source 

    type SourceLinkedList
        type(Source), pointer :: firstSource => Null()
        type(Source), pointer :: currentSource => NULL()
        
    end type SourceLinkedList

public AddSourceToList
public DeleteSourceList
contains

!********************************
! AddSourceToList
!********************************
subroutine AddSourceToList(sourceList, data)
    type(SourceLinkedList), intent(inout) :: sourceList
   
    type(SourceParameters), intent(in) :: data
   
    if (.not. associated(sourceList%firstSource)) then
      
        sourceList%firstSource => &
            AllocateSource("sourceList%firstSource in omsourceparam_mod.f90")
        sourceList%firstSource%data = data
        sourceList%currentSource => sourceList%firstSource
    else

        sourceList%currentSource%next => &
            AllocateSource("sourceList%currentSource%next in omsourceparam_mod.f90")
        sourceList%currentSource => sourceList%currentSource%next
        sourceList%currentSource%data = data
         
    end if


end subroutine AddSourceToList    
!********************************
! DeleteSourceList
!********************************
subroutine DeleteSourceList(sourceList)
    type(SourceLinkedList), intent(inout) :: sourceList
    type(Source), pointer :: sourcePtr
    type(Source), pointer :: oldSource

    sourcePtr => sourceList%firstSource
   
    do while(associated(sourcePtr))


        oldSource=>sourcePtr
        sourcePtr=>sourcePtr%next

        deallocate(oldSource)
        nullify(oldSource)
    end do

    sourceList%firstSource => Null()
end subroutine DeleteSourceList         

!***************************************
! GetNextSourceData
!***************************************
logical function GetNextSourceData(index, firstSource, data)

    integer(int32), intent(inout) :: index
    type(Source), pointer :: firstSource
    type(SourceParameters), intent(out) :: data
    type(Source), pointer, save :: sourceList => Null()    

    if (index .eq. 1) then

        if (associated(firstSource)) then
            data = firstSource%data
            sourceList => firstSource%next
            index = index + 1
            GetNextSourceData = .true.
            
        else 
            GetNextSourceData = .false.
        end if

        return
    end if 


    if(associated(sourceList)) then
        data = sourceList%data
        sourceList => sourceList%next
        index = index + 1
        GetNextSourceData = .true.
    else
        GetNextSourceData = .false.

    end if


end function GetNextSourceData



!************************************************
! Function SourceDataArray
!
!*************************************************
function SourceDataArray1(sourceList, nSources) result(sourceData)
    type(SourceLinkedList), intent(inout):: sourceList
    integer(int32), intent(inout) :: nSources
    type(SourceParameters), pointer :: sourceData(:)
  
    integer(int32) :: index, k
    logical :: status

    !*******************************************************
    ! Copy the contents of the list to an array
    !*******************************************************
    nullify(sourceData)
    if (nSources .eq. 0) return 
    allocate(sourceData( 1 : nSources))
        
    index = 1    

    do k = 1, nSources
   
        status = GetNextSourceData(index, sourceList%firstSource, sourceData(k))
     
        if (.not. status) then 
            nSources = k - 1
            write(*, *) "No of sources = ", nSources, " k=", k
            call warning("Possible Error!", "nsources ne liinked-list size!")
            exit
        end if
    end do
end function SourceDataArray1

!************************************************
! Function SourceDataArray
!
!*************************************************
function SourceDataArray2(sourceList, nSources) result(sourceData)
    type(SourceLinkedList), intent(inout):: sourceList
    integer(int16), intent(inout) :: nSources
    type(SourceParameters), pointer :: sourceData(:)
  
    integer(int32) :: index, k
    logical :: status

    !*******************************************************
    ! Copy the contents of the list to an array
    !*******************************************************
    nullify(sourceData)
    if (nSources .eq. 0) return 
    allocate(sourceData( 1 : nSources))
        
    index = 1    

    do k = 1, nSources
   
        status = GetNextSourceData(index, sourceList%firstSource, sourceData(k))
  
        if (.not. status) then   
            write(*, *) "No of sources = ", nSources, " k=", k
            call warning("Possible Error!", "nsources ne liinked-list size!")
            exit
  
        end if
    end do
end function SourceDataArray2


!***********************************************
! ComputeSourceSignificance
! Computes the source significance and returns
! the value via the name
!***********************************************
real(single) function ComputeSourceSignificance1(sourceCounts, backGroundCounts)
    real(single), intent(in) :: sourceCounts     ! The background-subtracted source counts
    real(single), intent(in) :: backGroundCounts ! The background counts

    if (sourceCounts > 0.0) then 
        ComputeSourceSignificance1 = sourceCounts  / sqrt(sourceCounts + 2.0 * backGroundCounts)
    else
        ComputeSourceSignificance1 = 0.0_double
    end if
!write(*, *) "computesourcesignificance1 sourcecounts=", sourceCounts, &
!" background=", backgroundcounts, " sig=",  ComputeSourceSignificance1   

end function ComputeSourceSignificance1

!***********************************************
! ComputeSourceSignificance
! Computes the source significance and returns
! the value via the name
!***********************************************
real(single) function ComputeSourceSignificance2(sourceRate, backGroundRate, exposure)
    real(single), intent(in) :: sourceRate     ! The background-subtracted source counts
    real(single), intent(in) :: backGroundRate ! The background counts
    real(single), intent(in) :: exposure ! The background counts


    if (sourceRate > 0.0) then 
        ComputeSourceSignificance2 = sourceRate * exposure  / &
           sqrt((sourceRate + 2.0 * backGroundRate) * exposure)
    else
            ComputeSourceSignificance2 = 0.0_double
    end if
   
!write(*, *) "computesourcesignificance2 sourcereate=", sourceRate, &
!" background=", backgroundRate, " sig=",  ComputeSourceSignificance2, &
!" exposure=", exposure   
end function ComputeSourceSignificance2

!******************************************************************************
!
!
!******************************************************************************

function AllocateSource(name) result(newSource)
    character(*), intent(in) :: name
    character(100) :: comment
    type (Source), pointer :: newSource
    integer :: status 
    allocate(newSource, stat=status)
    
    if(status .ne. 0) then
       write(comment, '(a32, a20)') "Could not allocate memory for : ", name 
       
       call fatal("memory allocation error", comment) 
    end if
  


end function AllocateSource


end module omsourceparam_mod












